%% Parameters
% Household Parameters
par.beta        = 0.9795;     % Discount factor
par.xi          = 4;          % CRRA
par.gamma       = 1;          % Inverse Frisch elasticity
par.sigma_chi   = 22500;      % Variance of logit distribution of adj costs
par.logitratio  = 2.825;    
par.mu_chi      = par.logitratio*par.sigma_chi; % Mean of logit distribution of adj costs
par.nu          = 1;          % Prob. of trade given adj. decision
par.guessadj    = par.nu*1/(1+exp(par.mu_chi/par.sigma_chi));

% Income Process
par.rhoH        = 0.979;    % Persistence of productivity
par.sigmaH      = 0.059;    % STD of productivity shocks
par.rhoS        = 0.839;    % Persistence of variance
par.sigmaS      = 0.539;    % STD of variance shocks
mpar.in         = 0.0005;  % Prob. to become entrepreneur
mpar.out        = 0.0625;   % Prob. to become worker again

% Firm Side Parameters
par.eta         = 20;
par.mu          = (par.eta-1)/par.eta;       % Markup 5%
par.alpha       = 2/3/par.mu;  % Labor share 2/3
par.delta       = 0.054/4;     % Depreciation rate
par.phi         = 11.4;        % Capital adj costs

% Phillips Curve
par.prob_priceadj = 3/4; % average price duration of 4 quarters = 1/(1-par.prob_priceadj)
par.kappa         = (1-par.prob_priceadj)*(1-par.prob_priceadj*par.beta)/par.prob_priceadj;% Phillips-curve parameter (from Calvo prob.)
par.beta_PC       = par.beta;

% Central Bank Policy
par.theta_pi    = 1.25; % Reaction to inflation
par.rho_R       = 0.8;  % Inertia

% Tax Schedule
par.tau         = 0.7;   % Proportional tax on labor and profit income 

% Debt rule
par.gamma_pi    = 1.5;   % Reaction to inflation
par.gamma_T     = 0.5075; % Reaction to tax revenue
par.rho_B       = 0.86;  % Autocorrelation

%% Returns
par.PI  = 1.00^.25;     % Gross inflation
par.RB  = (par.PI*1.025)^0.25;    % Real return times inflation

par.ABS = 0;                    % Loan to value ratio max.
par.mortwedge = par.PI*(1.01^0.25-1);   % Wedge on mortgages
par.borrwedge = par.PI*(1.11^0.25-1); % Wedge on borrowing beyond secured borrowing

par.Q  = 1*(1-par.ABS);

%% Grids
% Idiosyncratic States
mpar.nm         = 80;
mpar.nk         = 80;
mpar.nh         = 26;
mpar.tauchen    ='importance';

% Aggr States
mpar.nK         = 1;
mpar.nM         = 1;
mpar.ns         = 1;


%% Numerical Parameters
mpar.crit    = 1e-10;
mpar.overrideEigen=true;

