% Steady state statistics
%%

[mesh.k,mesh.m]=meshgrid(grid.k,grid.m);
clear targets
targets.ShareBorrower=sum((grid.m<0).*sum(sum(joint_distr,2),3)');
targets.K=sum(grid.k.*sum(sum(joint_distr,1),3));
targets.B=grid.m*sum(sum(joint_distr,2),3)-par.ABS*targets.K;
Liquid  = max(min(mesh.m,0),mesh.m-mesh.k*par.ABS);
Illiquid =(mesh.k*par.Q+min(max(mesh.m,0),mesh.k*par.ABS));
Mortgage = max(0,mesh.k*par.ABS-max(mesh.m,0));
JDredux = sum(joint_distr,3);
targets.LI=sum(JDredux(:).*Liquid(:));
targets.IL=sum(JDredux(:).*Illiquid(:));
targets.BoverK     = targets.B/targets.K;
targets.LIoverIL   = targets.LI/targets.IL;
targets.mortgShare = sum(JDredux(:).*Mortgage(:))./targets.K;
targets.AdjProb=AProb(:)'*joint_distr(:);
targets.StdAdjProb=(sum((AProb(:)).^2.*joint_distr(:))-targets.AdjProb.^2)^0.5;

targets.L=grid.N*sum(grid.h'.*squeeze(sum(sum(joint_distr,1),2)));
targets.KY=targets.K/Output;
targets.BY=targets.B/Output;
targets.LY=targets.L/Output;
targets.Y=Output;
BCaux_M=sum(sum(joint_distr,2),3);
targets.m_bc=BCaux_M(1,:);
targets.m_0=BCaux_M(grid.m==0);
BCaux_K=sum(sum(joint_distr,1),3);
targets.k_bc=BCaux_K(:,1);
aux_MK=sum(joint_distr,3);
targets.WtH_b0=sum(aux_MK(Liquid==0 & Illiquid>0));
targets.WtH_bnonpos=sum(aux_MK(Liquid<=0 & Illiquid>0));

targets.T =(1-par.tau).*W_fc.*grid.N +(1-par.tau).*Profits_fc;
par.gamma1=targets.B*(1-par.RB/par.PI)+targets.T;
par.R=R_fc - ((par.RB + par.mortwedge)./par.PI-1).*par.ABS;
par.W=W_fc(1);
par.PROFITS=Profits_fc(1);
par.N=grid.N;
targets.GtoY=par.gamma1/Output;

%% Ginis
% Net worth Gini
mplusk=mesh.k(:)*par.Q+mesh.m(:);
[mplusk, IX]       = sort(mplusk);
moneycapital_pdf   = JDredux(IX);
moneycapital_cdf   = cumsum(moneycapital_pdf);
targets.NegNetWorth= sum((mplusk<0).*moneycapital_pdf);

S                  = cumsum(moneycapital_pdf.*mplusk)';
S                  = [0 S];
targets.GiniW      = 1-(sum(moneycapital_pdf.*(S(1:end-1)+S(2:end))')/S(end));

% Liquid Gini
[liquid_sort, IX]  = sort(Liquid(:));
liquid_pdf         = JDredux(IX);
liquid_cdf         = cumsum(liquid_pdf);
targets.Negliquid  = sum((liquid_sort<0).*liquid_pdf);

S                  = cumsum(liquid_pdf.*liquid_sort)';
S                  = [0 S];
targets.GiniLI      = 1-(sum(liquid_pdf.*(S(1:end-1)+S(2:end))')/S(end));

% Illiquid Gini
[illiquid_sort, IX] = sort(Illiquid(:));
illiquid_pdf        = JDredux(IX);
illiquid_cdf        = cumsum(illiquid_pdf);
targets.Negliquid   = sum((illiquid_sort<0).*illiquid_pdf);

S                   = cumsum(illiquid_pdf.*illiquid_sort)';
S                   = [0 S];
targets.GiniIL      = 1-(sum(illiquid_pdf.*(S(1:end-1)+S(2:end))')/S(end));

%%   MPCs
[meshes.m,meshes.k,meshes.h] = ndgrid(grid.m,grid.k,grid.h);

NW=par.gamma/(1+par.gamma).*(par.N/par.H).*par.W;
WW=NW*ones(mpar.nm,mpar.nk,mpar.nh); %Wages
WW(:,:,end)=par.PROFITS*par.profitshare;
% MPC
WW_h=squeeze(WW(1,1,:));
WW_h_mesh=squeeze(WW(:,:,:).*meshes.h);

grid_h_aux=grid.h;

MPC_a_m = zeros(mpar.nm,mpar.nk,mpar.nh);
MPC_n_m = zeros(mpar.nm,mpar.nk,mpar.nh);
for kk=1:mpar.nk
    for hh=1:mpar.nh
        MPC_a_m(:,kk,hh)=gradient(squeeze(c_a_guess(:,kk,hh)))./gradient(grid.m)';
        MPC_n_m(:,kk,hh)=gradient(squeeze(c_n_guess(:,kk,hh)))./gradient(grid.m)';
    end
end

MPC_a_m=MPC_a_m.*(WW_h_mesh./c_a_guess);
MPC_n_m=MPC_n_m.*(WW_h_mesh./c_n_guess);

MPC_a_h = zeros(mpar.nm,mpar.nk,mpar.nh);
MPC_n_h = zeros(mpar.nm,mpar.nk,mpar.nh);
for mm=1:mpar.nm
    for kk=1:mpar.nk
        MPC_a_h(mm,kk,:)=gradient(squeeze(log(c_a_guess(mm,kk,:))))./gradient(log(WW_h'.*grid_h_aux))';
        MPC_n_h(mm,kk,:)=gradient(squeeze(log(c_n_guess(mm,kk,:))))./gradient(log(WW_h'.*grid_h_aux))';
    end
end

EMPC_h=joint_distr(:)'*(AProb(:).*MPC_a_h(:)+(1-AProb(:)).*MPC_n_h(:));
EMPC_m=joint_distr(:)'*(AProb(:).*MPC_a_m(:)+(1-AProb(:)).*MPC_n_m(:));

EMPC_a_h=joint_distr(:)'*MPC_a_h(:);
EMPC_a_m=joint_distr(:)'*MPC_a_m(:);

EMPC_n_h=joint_distr(:)'*MPC_n_h(:);
EMPC_n_m=joint_distr(:)'*MPC_n_m(:);

targets.Insurance_coeff=[1-EMPC_h 1-EMPC_m;
    1-EMPC_a_h 1-EMPC_a_m;
    1-EMPC_n_h 1-EMPC_n_m];

%% Adjustment costs
willingPay    = par.mu_chi - log((1-AProb)./AProb) * par.sigma_chi;
willingPay    = max(willingPay,0);% max to capture numerical imprecision. Should always be positive

if par.xi ~=1 % u(c_a *(1-tau)) = u(c_a) - chi_max; (1-tau).^(1-sigma) = 1 - chi_max/u(c_a)
    CEwillingPay  = 1-1./(1 + (par.xi-1)*willingPay.*(c_a_guess).^(par.xi-1)).^(1./(par.xi-1));
else
    CEwillingPay  = 1-exp(1-willingPay./log(c_a_guess))-1;
end

targets.ACEwillingPay = joint_distr(:)'*CEwillingPay(:);
[CEwilling_sort, IX]  = sort(CEwillingPay(:));
CEwillingPay_pdf         = joint_distr(IX);
CEwillingPay_cdf         = cumsum(CEwillingPay_pdf);
targets.MedianCEwillingPay = max(CEwilling_sort(CEwillingPay_cdf<=0.5));
targets.NinetyCEwillingPay = max(CEwilling_sort(CEwillingPay_cdf<=0.9));
%% Liquidity

c_0=AProb.*c_a_guess+(1-AProb).*c_n_guess;
m_0=AProb.*m_a_star+(1-AProb).*m_n_star;
k_0=AProb.*cap_a_star+(1-AProb).*meshes.k;
[lambdaLIP,lambdaILP,lambdaC,lambdaAP,lambdaLI,lambdaIL,locstdIL,locstdLI,lambdaLIINC] = diststats_liquidity(joint_distr,c_0,m_0,k_0,AProb,grid,mpar,par,par.Q);

lowcut=21;


figurename=['Model_Average_LiquidityBAR_' casename '_ABS_' num2str(par.ABS*100)];
figure('Name',figurename,'NumberTitle','off','Position',[1 1 700 500])
aux=(lambdaLI(21:end,:)./lambdaIL(21:end,:))*100;
LIRATIOSQUINTILES=mean(reshape(aux,[20 4]),1);
bar(2:1:5,LIRATIOSQUINTILES);
ylabel('Liquid to illiquid assets (%)','Interpreter','none','FontName','arial','FontSize',20)
xlabel('Quintile of net worth','Interpreter','none','FontName','arial','FontSize',20)
hold on
set(gca, 'FontName','arial','FontSize',20);
set(gca,'Position',[0.125 0.15 0.78 0.8])
printpdf(gcf,['latex/' figurename])

targets.LIratiomedian=mean(lambdaLI([51],:)./lambdaIL([51],:));

targets

%% Moments Table

filename=['latex/assetcalibration_' casename '_ABS_' num2str(par.ABS*100) '.tex'];
FID = fopen(filename, 'w');
fprintf(FID, '\\begin{table}[th!] \n');
fprintf(FID, '\\begin{center} \n');
fprintf(FID, '\\caption{Moments targeted in calibration}\\label{tab:moments} \n');
fprintf(FID, ' \\begin{tabular}{l cccl} \n');

fprintf(FID, '\\hline \\hline \n');
fprintf(FID, '\\\\ Targets & Model & Data &  Source & Parameter \\\\ \\hline \n');

fprintf(FID, '\\\\ Mean illiquid assets (K/Y)  & %8.2f  & 2.86   &    NIPA  & Discount factor  \n', targets.KY/4);
fprintf(FID, '\\\\ Mean liquidity (B/K)        & %8.2f  & 0.09 &   SCF & Mean adj costs    \n', targets.LIoverIL);
fprintf(FID, '\\\\ 2nd quintile liquidity (b/k)       & %8.2f  & 0.33   &   SCF & Variance adj costs    \n', LIRATIOSQUINTILES(1)/100);
fprintf(FID, '\\\\ Gini total wealth           & %8.2f  & 0.78   & SCF & Fraction Entrepreneurs      \n', targets.GiniW);
fprintf(FID, '\\\\ Fraction borrowers          & %8.2f  & 0.16   & SCF & Borrowing penalty      \n', targets.ShareBorrower);

fprintf(FID, '\\\\ \\hline \n');


fprintf(FID, '\\end{tabular} \n');
fprintf(FID, '\\end{center} \n');
fprintf(FID, '\\end{table} \n');

fclose(FID);