function [c_n_guess,m_n_star,c_a_guess,m_a_star,cap_a_star,psi_guess,joint_distr, AProb,V,R_fc,W_fc,Profits_fc,Output,grid]=steadystate(P_H,grid,mpar,par)
  %     Prepare items for EGM
  %     Layout of matrices:
  %       Dimension 1: money m
  %       Dimension 2: capital k
  %       Dimension 3: stochastic sigma s x stochastic human capital h
  %       Dimension 4: aggregate money M
  %       Dimension 5: aggregate capital K

[meshes.m,meshes.k,meshes.h] = ndgrid(grid.m,grid.k,grid.h);

% 1) Construct relevant return matrices
AProb=par.guessadj*ones(mpar.nm,mpar.nk,mpar.nh);
joint_distr = ones(mpar.nm,mpar.nk,mpar.nh)/(mpar.nh*mpar.nk*mpar.nm);

[ ~,~,~,~,WW,RR,RBRB,~] = factor_returns(meshes,grid,par,mpar);

% 2) Guess initial policies
[c_a_guess,c_n_guess, psi_guess,~]=policyguess(meshes,WW,RR,RBRB,par,mpar);

KL=0.75*grid.K;
KH=1.25*grid.K;

[excessL]=excessK(KL,c_a_guess,c_n_guess,psi_guess,AProb,joint_distr, grid,P_H,mpar,par,meshes);

[excessH]=excessK(KH,c_a_guess,c_n_guess,psi_guess,AProb,joint_distr, grid,P_H,mpar,par,meshes);

if sign(excessL)==sign(excessH)
    disp('ERROR! Sign not diff')
end

%% Brent
fa=excessL; fb=excessH;
a=KL;
b=KH;

if fa*fb>0
    error('f(a) und f(b) sollten unterschiedliche Vorzeichen haben');
end

c=a; fc=fa;   %At the beginning: c = a

c=a; fc=fa; d=b-a; e=d;

iter=0;
maxiter=1000;

while iter<maxiter
    iter=iter+1
    
    if fb*fc>0
        c=a; fc=fa; d=b-a; e=d;
    end
    
    if abs(fc)<abs(fb)
        a=b; b=c; c=a;
        fa=fb; fb=fc; fc=fa;
    end
    
    tol=2*eps*abs(b)+mpar.crit; m=(c-b)/2; % tolerance
    
    if (abs(m)>tol) && (abs(fb)>0)
        
        if (abs(e)<tol) || (abs(fa)<=abs(fb))
            d=m; e=m;
        else
            s=fb/fa;
            if a==c
                p=2*m*s; q=1-s;
            else
                q=fa/fc; r=fb/fc;
                p=s*(2*m*q*(q-r)-(b-a)*(r-1));
                q=(q-1)*(r-1)*(s-1);
            end
            if p>0
                q=-q;
            else
                p=-p;
            end
            s=e; e=d;
            if ( 2*p<3*m*q-abs(tol*q) ) && (p<abs(s*q/2))
                d=p/q;
            else
                d=m; e=m;
            end
        end
        
        a=b; fa=fb;
        
        if abs(d)>tol
            b=b+d;
        else
            if m>0
                b=b+tol;
            else
                b=b-tol;
            end
        end
    else
        break;
    end
    
    [fb,c_n_guess,~,c_a_guess,~,~,psi_guess,joint_distr, AProb,~]=excessK(b,c_a_guess,c_n_guess,psi_guess,AProb,joint_distr, grid,P_H,mpar,par,meshes);
    
end

Kcand=b;
grid.K=b;

%% Update

[excess,c_n_guess,m_n_star,c_a_guess,m_a_star,cap_a_star,psi_guess,joint_distr, AProb,V,R_fc,W_fc,Profits_fc,Output,N]=excessK(Kcand,c_a_guess,c_n_guess,psi_guess,AProb,joint_distr, grid,P_H,mpar,par,meshes);

grid.N=N;
end
function [excess,c_n_guess,m_n_star,c_a_guess,m_a_star,cap_a_star,psi_guess,joint_distr, AProb_new,V,R_fc,W_fc,Profits_fc,Output,N]=excessK(K,c_a_guess,c_n_guess,psi_guess,AProb,joint_distr, grid,P_H,mpar,par,meshes)
dist_AP=9999;
distPOL=9999;
grid.K=K;

[ N,R_fc,W_fc,Profits_fc,WW,RR,RBRB,Output] = factor_returns(meshes,grid,par,mpar);

[c_a_guess,c_n_guess, psi_guess,inc]=policyguess(meshes,WW,RR,RBRB,par,mpar);
count=0;

while max(dist_AP)>mpar.crit && count<3
    count=count+1;
    % 3) Solve Policies and Joint Distribution
    disp('Solving household problem by EGM')
    tic
    [c_n_guess,m_n_star,c_a_guess,m_a_star,cap_a_star,psi_guess,distPOL]=...
        policies_SS(c_a_guess,c_n_guess,psi_guess,AProb, grid, inc, RR,RBRB,P_H,mpar,par,meshes);
    toc
    
    disp('Calc Value Function and AProb')
    tic
    [AProb_new,V,H] = ValueFunctionSS(m_n_star,m_a_star,cap_a_star,c_a_guess,c_n_guess,AProb,P_H,mpar,par,grid);
    joint_distr   = reshape(joint_distr,[mpar.nm,mpar.nk, mpar.nh]);
    toc
    
    dist_AP=max(abs(AProb_new(:)-AProb(:)));
    AProb=AProb_new;
    
end
    
disp(([distPOL dist_AP]));

disp('Calc Joint Distr')

[joint_distr]=JDiteration(joint_distr,H,AProb,m_n_star,m_a_star,cap_a_star,P_H,mpar,grid);
joint_distr=reshape(joint_distr,[mpar.nm mpar.nk mpar.nh]);
AggregateCapitalDemand=sum(grid.k.*sum(sum(joint_distr,1),3));
excess=(grid.K-AggregateCapitalDemand);
end
