function [cap_n_partial,m_a_partial,m_n_partial,cap_a_partial,c_a_partial,c_n_partial,AProb_partial]=partial_liquidity(AProb,psi_guess,c_a_guess,c_n_guess,joint_distr,V,P_H,par,mpar,grid)

%% Stochastic transition in s
% Determine number of gridpoints for grid.s
mpar.ns=5;
mpar.nH=5; % HH = 2*log(H)= sigma^2_{h,t} = par.rho_h^2 * sigma^2_{sh,t-1} + exp(s)
% HH(t) = par.rho_h^2 HH(t-1) + exp(s)
% H(t)  = exp(1/2 HH(t));

% Generate transition probabilities and grid
[sgrid,P_S]  = Tauchen(par.rhoS,mpar.ns,1, 0, 'importance'); % LR variance = 1
sgrid        = sgrid*par.sigmaS/sqrt(1-par.rhoS^2);
grid.s       = exp(sgrid)/mean(exp(sgrid));

maxHH        = (1+1-grid.s(1))*log(par.H);
minHH        = grid.s(1)*log(par.H);
grid.HH      = linspace(-maxHH,-minHH,mpar.nH);%1./(1-par.rhoH^2)+linspace(-2*par.xiS/sqrt(factor^2), 2*par.xiS/sqrt(factor^2),mpar.nH);
[mH,ms]      = ndgrid(grid.HH,(-grid.s'));
Hplus        = par.rhoH^2*mH + par.sigmaH^2/2 * ms;
Hplus        = exp(Hplus);
grid.H       = exp(grid.HH);%+(par.H-exp(mean_HH/2));
[Dist_H,idH] = genweight(Hplus,grid.H);
P_HH         = zeros(mpar.nH,mpar.nH,mpar.ns);
for j=1:mpar.ns
    for i=1:mpar.nH
        P_HH(i,idH(i,j)+1,j) = Dist_H(i,j);
        P_HH(i,idH(i,j),j)   = 1-Dist_H(i,j);
    end
end

[~,~,boundsH] = Tauchen(par.rhoH,mpar.nh-1,1, 0, 'importance'); % LR variance = 1

P_aux=repmat (P_H,[1 1 mpar.ns]);

for j = 1:mpar.ns
    aux                    = sqrt(grid.s(j))*sqrt(1-par.rhoH^2); % Short run std
    P_aux(1:end-1,1:end-1,j) = transition(mpar.nh-1,par.rhoH,aux,boundsH);
end
P_aux=P_aux./repmat(sum(P_aux,2),[1 size(P_aux,2)]);

% Define indeces for converting joint states to individual ones
h_index=repmat((1:mpar.nh),mpar.ns,1);
h_index=h_index(:); % maps joint states to h dimension
s_index=repmat((1:mpar.ns)',1,mpar.nh);
s_index=s_index(:); % maps joint states to s dimension

% Calculate transition probability matrix for joint process
P=NaN(mpar.ns*mpar.nh*mpar.nH);
for HH=1:mpar.nH
    for hh=1:mpar.nh
        for s=1:mpar.ns
            P(s + (hh-1)*mpar.ns+(HH-1)*mpar.ns*mpar.nh,:)=kron(kron(P_HH(HH,:,s),P_aux(hh,:,s)),P_S(s,:));
        end
    end
end
P=P./repmat(sum(P,2),[1 size(P,2)]);

shgrid=repmat(grid.h,[mpar.ns*mpar.nH 1]);
shgrid=shgrid(:)';

%% Find policies for grid.s, P_S, and given prices

[meshes.m,meshes.k,meshes.h] = ndgrid(grid.m,grid.k,shgrid);

[meshaux.m,meshaux.k,meshaux.h] = ndgrid(grid.m,grid.k,1:length(shgrid));


mc =  par.mu - (par.beta * log(par.PI) - log(par.PI))/par.kappa;

%%  GHH Preferences
N    =  (par.tau.*par.alpha.*grid.K.^(1-par.alpha).*mc).^(1/(1-par.alpha+par.gamma));
W_fc = par.alpha .*mc.* (grid.K./N).^(1-par.alpha);

% Before tax return on capital
R_fc = (1-par.alpha) .*mc.* (N./grid.K).^(par.alpha)- par.delta;

Y = (N).^(par.alpha).*grid.K.^(1-par.alpha);
Profits_fc = (1-mc)*Y;

NW=par.gamma/(1+par.gamma).*(N).*W_fc;
WW=NW*ones(mpar.nm,mpar.nk,mpar.ns*mpar.nh*mpar.nH); %Wages
WW(:,:,end)=Profits_fc*par.profitshare;
RR = R_fc - ((par.RB + par.mortwedge)./par.PI-1).*par.ABS; %Rental rates
RBRB = (par.RB+(meshes.m<0)*par.borrwedge + par.mortwedge.*(meshes.m<par.ABS.*meshes.k))./par.PI;

meshHH=permute(repmat(grid.H,[mpar.ns 1 mpar.nh]),[1 3 2]);
meshHH=permute(repmat(meshHH(:), [1 mpar.nm mpar.nk]),[2 3 1]);

inc.labor   = par.tau.*WW.*meshes.h./meshHH;
inc.rent    = RR.*meshes.k;
inc.money   = RBRB.*meshes.m;
inc.capital = par.Q.*meshes.k;
inc.profits = 0;

AProb=reshape(permute(repmat(AProb,[1 1 1 mpar.ns*mpar.nH]),[1 2 4 3]),[mpar.nm mpar.nk mpar.ns*mpar.nh*mpar.nH]);
psi_guess=reshape(permute(repmat(psi_guess,[1 1 1 mpar.ns*mpar.nH]),[1 2 4 3]),[mpar.nm mpar.nk mpar.ns*mpar.nh*mpar.nH]);
c_a_guess=reshape(permute(repmat(c_a_guess,[1 1 1 mpar.ns*mpar.nH]),[1 2 4 3]),[mpar.nm mpar.nk mpar.ns*mpar.nh*mpar.nH]);
c_n_guess=reshape(permute(repmat(c_n_guess,[1 1 1 mpar.ns*mpar.nH]),[1 2 4 3]),[mpar.nm mpar.nk mpar.ns*mpar.nh*mpar.nH]);
V=reshape(permute(repmat(V,[1 1 1 mpar.ns*mpar.nH]),[1 2 4 3]),[mpar.nm mpar.nk mpar.ns*mpar.nh*mpar.nH]);

%%
util  = @(c)(c.^(1-par.xi))./(1-par.xi);

count=0;
dist_AP=999;
while dist_AP>mpar.crit && count<10
    count=count+1;
    % 5) Solve Policies and Joint Distribution
    disp('Solving household problem by EGM')
    mpar.nh=mpar.nh*mpar.nH;
    tic
    [c_n_guess,m_n_star,c_a_guess,m_a_star,cap_a_star,psi_guess,distPOL]=...
        policies_SS(c_a_guess,c_n_guess,psi_guess,AProb, grid, inc, RR,RBRB,P,mpar,par,meshes);
    toc
    mpar.nh=mpar.nh/mpar.nH;
    
    AC =  par.sigma_chi.*((1-AProb(:)).*log(1-AProb(:)) + AProb(:).*log(AProb(:)))...
        + par.mu_chi.*AProb(:) -  (par.mu_chi-par.sigma_chi*log(1+exp(par.mu_chi/par.sigma_chi)));
    AC    = AC.*par.nu;
    
    
    disp('Calc Value Function and AProb')
    tic
    countV=0;
    dist_V=999;
    while dist_V>mpar.crit && countV<1000
        EV    = reshape(V,[mpar.nm*mpar.nk, mpar.nh*mpar.ns*mpar.nH])*P';
        Vnext = griddedInterpolant(meshaux.m,meshaux.k,meshaux.h,reshape(EV,[mpar.nm mpar.nk mpar.nh*mpar.ns*mpar.nH]));
        
        VA    = util(c_a_guess) + par.beta*Vnext(m_a_star,cap_a_star,meshaux.h);
        VN    = util(c_n_guess) + par.beta*Vnext(m_n_star,meshaux.k,meshaux.h);
        
        Vnew  = AProb(:).*VA(:) + (1-AProb(:)).*VN(:) - AC(:); % Calculate new Value Function
        dist_V=max(abs(Vnew(:)-V(:)));
        V=Vnew;
        countV=countV+1;
    end
    
    AProb_new = max(min(1./(1+exp(-((VA-VN)-par.mu_chi)./par.sigma_chi)),1-1e-6),1e-6);
    dist_AP=max(abs(AProb_new(:)-AProb(:)));
    AProb=AProb_new;
    
    toc
    
end

cap_n_partial=repmat(grid.k,[mpar.nm 1 mpar.ns mpar.nh mpar.nH]);
m_a_partial= reshape(m_a_star, [mpar.nm mpar.nk mpar.ns mpar.nh mpar.nH]);
m_n_partial= reshape(m_n_star, [mpar.nm mpar.nk mpar.ns mpar.nh mpar.nH]);
cap_a_partial= reshape(cap_a_star, [mpar.nm mpar.nk mpar.ns mpar.nh mpar.nH]);
c_a_partial= reshape(c_a_guess, [mpar.nm mpar.nk mpar.ns mpar.nh mpar.nH]);
c_n_partial= reshape(c_n_guess, [mpar.nm mpar.nk mpar.ns mpar.nh mpar.nH]);
AProb_partial= reshape(AProb, [mpar.nm mpar.nk mpar.ns mpar.nh mpar.nH]);


%% Calculate consumption, money and capital holdings for each percentile after 1 STD shock

[~,id1]=histc(1,grid.s);
id1(1>=grid.s(end)) = mpar.ns-1;
id1(1<=grid.s(1))   = 1;

dist1=(1- grid.s(id1))/(grid.s(id1+1) - grid.s(id1));

moneydemand_A_0=squeeze(m_a_partial(:,:,id1,:,ceil(mpar.nH/2)))+ ...
    dist1*(squeeze(m_a_partial(:,:,id1+1,:,ceil(mpar.nH/2)))- squeeze(m_a_partial(:,:,id1,:,ceil(mpar.nH/2))));
capitaldemand_A_0=squeeze(cap_a_partial(:,:,id1,:,ceil(mpar.nH/2)))+ ...
    dist1*(squeeze(cap_a_partial(:,:,id1+1,:,ceil(mpar.nH/2)))- squeeze(cap_a_partial(:,:,id1,:,ceil(mpar.nH/2))));
consumptiondemand_A_0=squeeze(c_a_partial(:,:,id1,:,ceil(mpar.nH/2)))+ ...
    dist1*(squeeze(c_a_partial(:,:,id1+1,:,ceil(mpar.nH/2)))- squeeze(c_a_partial(:,:,id1,:,ceil(mpar.nH/2))));

moneydemand_N_0=squeeze(m_n_partial(:,:,id1,:,ceil(mpar.nH/2)))+ ...
    dist1*(squeeze(m_n_partial(:,:,id1+1,:,ceil(mpar.nH/2)))- squeeze(m_n_partial(:,:,id1,:,ceil(mpar.nH/2))));
capitaldemand_N_0=squeeze(cap_n_partial(:,:,id1,:,ceil(mpar.nH/2)))+ ...
    dist1*(squeeze(cap_n_partial(:,:,id1+1,:,ceil(mpar.nH/2)))- squeeze(cap_n_partial(:,:,id1,:,ceil(mpar.nH/2))));
consumptiondemand_N_0=squeeze(c_n_partial(:,:,id1,:,ceil(mpar.nH/2)))+ ...
    dist1*(squeeze(c_n_partial(:,:,id1+1,:,ceil(mpar.nH/2)))- squeeze(c_n_partial(:,:,id1,:,ceil(mpar.nH/2))));

AProb_aux_0=squeeze(AProb_partial(:,:,id1,:,ceil(mpar.nH/2)))+ ...
    dist1*(squeeze(AProb_partial(:,:,id1+1,:,ceil(mpar.nH/2)))- squeeze(AProb_partial(:,:,id1,:,ceil(mpar.nH/2))));


STD2=1+par.sigmaS;
[~,id2]=histc(STD2,grid.s);
id2(STD2>=grid.s(end)) = mpar.ns-1;
id2(STD2<=grid.s(1))   = 1;

dist2=(STD2 - grid.s(id2))/(grid.s(id2+1) - grid.s(id2));

P_H_1=P_aux(:,:,id2)+ ...
    dist2*(P_aux(:,:,id2+1)- P_aux(:,:,id2));

moneydemand_A_1=squeeze(m_a_partial(:,:,id2,:,ceil(mpar.nH/2)))+ ...
    dist2*(squeeze(m_a_partial(:,:,id2+1,:,ceil(mpar.nH/2)))- squeeze(m_a_partial(:,:,id2,:,ceil(mpar.nH/2))));
capitaldemand_A_1=squeeze(cap_a_partial(:,:,id2,:,ceil(mpar.nH/2)))+ ...
    dist2*(squeeze(cap_a_partial(:,:,id2+1,:,ceil(mpar.nH/2)))- squeeze(cap_a_partial(:,:,id2,:,ceil(mpar.nH/2))));
consumptiondemand_A_1=squeeze(c_a_partial(:,:,id2,:,ceil(mpar.nH/2)))+ ...
    dist2*(squeeze(c_a_partial(:,:,id2+1,:,ceil(mpar.nH/2)))- squeeze(c_a_partial(:,:,id2,:,ceil(mpar.nH/2))));

moneydemand_N_1=squeeze(m_n_partial(:,:,id2,:,ceil(mpar.nH/2)))+ ...
    dist2*(squeeze(m_n_partial(:,:,id2+1,:,ceil(mpar.nH/2)))- squeeze(m_n_partial(:,:,id2,:,ceil(mpar.nH/2))));
capitaldemand_N_1=squeeze(cap_n_partial(:,:,id2,:,ceil(mpar.nH/2)))+ ...
    dist2*(squeeze(cap_n_partial(:,:,id2+1,:,ceil(mpar.nH/2)))- squeeze(cap_n_partial(:,:,id2,:,ceil(mpar.nH/2))));
consumptiondemand_N_1=squeeze(c_n_partial(:,:,id2,:,ceil(mpar.nH/2)))+ ...
    dist2*(squeeze(c_n_partial(:,:,id2+1,:,ceil(mpar.nH/2)))- squeeze(c_n_partial(:,:,id2,:,ceil(mpar.nH/2))));

AProb_aux_1=squeeze(AProb_partial(:,:,id2,:,ceil(mpar.nH/2)))+ ...
    dist2*(squeeze(AProb_partial(:,:,id2+1,:,ceil(mpar.nH/2)))- squeeze(AProb_partial(:,:,id2,:,ceil(mpar.nH/2))));


%%
c_0=AProb_aux_0.*consumptiondemand_A_0+(1-AProb_aux_0).*consumptiondemand_N_0;
m_0=AProb_aux_0.*moneydemand_A_0+(1-AProb_aux_0).*moneydemand_N_0;
k_0=AProb_aux_0.*capitaldemand_A_0+(1-AProb_aux_0).*capitaldemand_N_0;

[lambdaLI_0,lambdaIL_0,lambdaC_0,lambdaAP_0] = diststats_liquidity(joint_distr,c_0,m_0,k_0,AProb_aux_0,grid,mpar,par,1);

c_1=AProb_aux_1.*consumptiondemand_A_1+(1-AProb_aux_1).*consumptiondemand_N_1;
m_1=AProb_aux_1.*moneydemand_A_1+(1-AProb_aux_1).*moneydemand_N_1;
k_1=AProb_aux_1.*capitaldemand_A_1+(1-AProb_aux_1).*capitaldemand_N_1;
[lambdaLI_1,lambdaIL_1,lambdaC_1,lambdaAP_1] = diststats_liquidity(joint_distr,c_1,m_1,k_1,AProb_aux_1,grid,mpar,par,1);

%% Plotting

figurename=['IRF_PortLiquidityPE'];
figure('Name',figurename,'NumberTitle','off','Position',[1 1 700 500])
plot(21:100,100*((lambdaLI_1(21:end)./lambdaIL_1(21:end))...
    ./(lambdaLI_0(21:end)./lambdaIL_0(21:end))-1),'LineWidth',3)

ylabel('Percent','Interpreter','none','FontName','arial','FontSize',20)
xlabel('Percentile of net worth','Interpreter','none','FontName','arial','FontSize',20)
hold on
set(gca, 'FontName','arial','FontSize',20);
set(gca,'Position',[0.175 0.15 0.78 0.8])
printpdf(gcf, ['latex\' figurename])

figurename=['IRF_MoneyPE'];
figure('Name',figurename,'NumberTitle','off','Position',[1 1 700 500])
plot(21:100,100*(lambdaLI_1(21:end)./lambdaLI_0(21:end)-1),'LineWidth',3)
ylabel('Percent','Interpreter','none','FontName','arial','FontSize',20)
xlabel('Percentile of net worth','Interpreter','none','FontName','arial','FontSize',20)
hold on
set(gca, 'FontName','arial','FontSize',20);
set(gca,'Position',[0.175 0.15 0.78 0.8])
printpdf(gcf, ['latex\' figurename])

figurename=['IRF_CapitalPE'];
figure('Name',figurename,'NumberTitle','off','Position',[1 1 700 500])
plot(21:100,100*(lambdaIL_1(21:end)./lambdaIL_0(21:end)-1),'LineWidth',3)
ylabel('Percent','Interpreter','none','FontName','arial','FontSize',20)
xlabel('Percentile of net worth','Interpreter','none','FontName','arial','FontSize',20)
hold on
set(gca, 'FontName','arial','FontSize',20);
set(gca,'Position',[0.175 0.15 0.78 0.8])
printpdf(gcf, ['latex\' figurename])

figurename=['IRF_ConsumptionPE'];
figure('Name',figurename,'NumberTitle','off','Position',[1 1 700 500])
plot(1:100,100*(lambdaC_1(1:end)./lambdaC_0(1:end)-1),'LineWidth',3)
ylabel('Percent','Interpreter','none','FontName','arial','FontSize',20)
xlabel('Percentile of net worth','Interpreter','none','FontName','arial','FontSize',20)
hold on
set(gca, 'FontName','arial','FontSize',20);
set(gca,'Position',[0.175 0.15 0.78 0.8])
printpdf(gcf, ['latex\' figurename])

%% adjust vs non-adjsut
c_0_A=AProb_aux_0.*consumptiondemand_A_0;
c_0_N=(1-AProb_aux_0).*consumptiondemand_N_0;
m_0_A=AProb_aux_0.*moneydemand_A_0;
m_0_N=(1-AProb_aux_0).*moneydemand_N_0;
k_0=AProb_aux_0.*capitaldemand_A_0+(1-AProb_aux_0).*capitaldemand_N_0;

[lambdaLI_0_A,~,lambdaC_0_A,~] = diststats_liquidity(joint_distr,c_0_A,m_0_A,k_0,AProb_aux_0,grid,mpar,par,1);
[lambdaLI_0_N,~,lambdaC_0_N,~] = diststats_liquidity(joint_distr,c_0_N,m_0_N,k_0,AProb_aux_0,grid,mpar,par,1);

c_1_A=AProb_aux_1.*consumptiondemand_A_1;
c_1_N=(1-AProb_aux_1).*consumptiondemand_N_1;
m_1_A=AProb_aux_1.*moneydemand_A_1;
m_1_N=(1-AProb_aux_1).*moneydemand_N_1;
k_1=AProb_aux_1.*capitaldemand_A_1+(1-AProb_aux_1).*capitaldemand_N_1;

[lambdaLI_1_A,~,lambdaC_1_A,~] = diststats_liquidity(joint_distr,c_1_A,m_1_A,k_1,AProb_aux_1,grid,mpar,par,1);
[lambdaLI_1_N,~,lambdaC_1_N,~] = diststats_liquidity(joint_distr,c_1_N,m_1_N,k_1,AProb_aux_1,grid,mpar,par,1);


figurename=['IRF_ConsumptionPE_ADJNON'];
figure('Name',figurename,'NumberTitle','off','Position',[1 1 700 500])
plot(1:100,100*(lambdaC_1_A(1:end)./lambdaC_0_A(1:end)-1),'--','LineWidth',3)
hold on
plot(1:100,100*(lambdaC_1_N(1:end)./lambdaC_0_N(1:end)-1),'LineWidth',3)
ylabel('Percent','Interpreter','none','FontName','arial','FontSize',20)
xlabel('Percentile of net worth','Interpreter','none','FontName','arial','FontSize',20)
legend({'adjust','non-adjust'},'Location','SouthEast')
set(gca, 'FontName','arial','FontSize',20);
set(gca,'Position',[0.175 0.15 0.78 0.8])
printpdf(gcf, ['latex\' figurename])

figurename=['IRF_MoneyPE_ADJNON'];
figure('Name',figurename,'NumberTitle','off','Position',[1 1 700 500])
plot(21:100,100*(lambdaLI_1_A(21:end)./lambdaLI_0_A(21:end)-1),'--','LineWidth',3)
hold on
plot(21:100,100*(lambdaLI_1_N(21:end)./lambdaLI_0_N(21:end)-1),'LineWidth',3)
ylabel('Percent','Interpreter','none','FontName','arial','FontSize',20)
xlabel('Percentile of net worth','Interpreter','none','FontName','arial','FontSize',20)
legend({'adjust','non-adjust'},'Location','NorthEast')
set(gca, 'FontName','arial','FontSize',20);
set(gca,'Position',[0.175 0.15 0.78 0.8])
printpdf(gcf, ['latex\' figurename])

end
%%
function [ weight,index ] = genweight( x,xgrid )
% function: GENWEIGHT generates weights and indexes used for linear interpolation
%
% X: Points at which function is to be interpolated.
% xgrid: grid points at which function is measured
% no extrapolation allowed
[~,index] = histc(x,xgrid);
index(x<=xgrid(1))=1;
index(x>=xgrid(end))=length(xgrid)-1;

weight = (x-xgrid(index))./(xgrid(index+1)-xgrid(index)); % weight xm of higher gridpoint
weight(weight<=0) = 1.e-16; % no extrapolation
weight(weight>=1) = 1-1.e-16; % no extrapolation

end  % funct
