%
% =========================================================================
% Part of the Matlab code to accompany the paper
% 'Precautionary Savings, Illiquid Assets, and the Aggregate Consequences of
%  Shocks to Household Income Risk', Bonn mimeo
% http://wiwi.uni-bonn.de/hump/wp.html
% =========================================================================
%
% Permission is hereby granted, free of charge, to any person obtaining a
% copy of this software and associated documentation files (the "Software"),
% to deal in the Software without restriction, including without limitation
% the rights to use, copy, modify, merge, publish, distribute, sublicense,
% and/or sell copies of the Software, and to permit persons to whom the
% Software is furnished to do so, subject to the following conditions:
%
% The most recent version or successor of the above-mentioned paper is
% properly cited in all work and publications that benefit from insights
% derived from the Software.
% The above copyright notice and this permission notice shall be included
% in all copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
% THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
% DEALINGS IN THE SOFTWARE.
%
% =========================================================================

%% Initialize workspace and load directories
clear
clc
close all
Computername='NKIM'
starttime=clock;
set(0,'defaulttextinterpreter','latex')
addpath(genpath('functions'))
addpath(genpath('latex'))

%% 1) Find baseline steady state
casename            = 'SS_BASELINE';
% Set parameters
defineSS_pars

mainskript_steadystate

%% 2) Produce baseline results for uncertainty shocks
aggrshock           = 'Uncertainty';
par.rhoS            = 0.839;    % Persistence of variance
par.sigmaS          = 0.539;    % STD of variance shocks

% Produce matrices to reduce state-space
mpar.maxdim=12;

mainskript_statereduc

% Initialize System
disp('Computing system for SS.');
F = @(a,b,c,d)Fsys(a,b,c,d,Xss,Yss,Gamma_state,Gamma_control,InvGamma,par,mpar,grid,targets,Copula,P_H,aggrshock,oc);
tic
[Fss,LHS,RHS,Distr] = F(State,State_m,Contr,Contr_m);
toc

% Solve RE via Schmitt-Grohe-Uribe Form
[hx,gx,F1,F2,F3,F4,par] = SGU_solver(F, numstates,numcontrols,Gamma_state,oc,mpar,par,grid);

% Produce IRFs
mpar.maxlag=40; % Quarters

x0=zeros(numstates,1);
x0(end)=par.sigmaS;

MX=[eye(length(x0));gx];
IRF_state_sparse=[];
x=x0;

for t=1:mpar.maxlag
    IRF_state_sparse(:,t)=(MX*x)';
    x=hx*x;
end

plot_IRFs_prep
save([casename '_IRF_data'],'IRF_Y','IRF_C','IRF_I','IRF_G','Deficit','IRF_K','IRF_M','IRF_N','IRF_S','IRF_PI','IRF_RB','IRF_RBREAL','IRF_Q','IRF_D','IRF_LP')

plot_IRFs

% Liquidity Response
% Partial Equilibrium Liquidity
[~]=partial_liquidity(AProb,psi_guess,c_a_guess,c_n_guess,joint_distr,V,P_H,par,mpar,grid);
% General Equilibrium Liquidity
GE_liquidity

% Welfare
welfare_skript

close all
%% 2) Produce results for uncertainty shocks and monetary stabilization
par.gamma_pi=0;
par.gamma_T=0;
par.theta_pi=100;

% Initialize System
disp('Computing system for SS.');
F = @(a,b,c,d)Fsys(a,b,c,d,Xss,Yss,Gamma_state,Gamma_control,InvGamma,par,mpar,grid,targets,Copula,P_H,aggrshock,oc);
tic
[Fss,LHS,RHS,Distr] = F(State,State_m,Contr,Contr_m);
toc

% Solve RE via Schmitt-Grohe-Uribe Form
[hx,gx,F1,F2,F3,F4,par] = SGU_solver(F, numstates,numcontrols,Gamma_state,oc,mpar,par,grid);

% Produce IRFs
mpar.maxlag=40; % Quarters

x0=zeros(numstates,1);
x0(end)=par.sigmaS;

MX=[eye(length(x0));gx];
IRF_state_sparse=[];
x=x0;

for t=1:mpar.maxlag
    IRF_state_sparse(:,t)=(MX*x)';
    x=hx*x;
end

plot_IRFs_prep

plot_IRFs

welfare_skript

close all
%% 2) Produce results for uncertainty shocks and fiscal stabilization
par.gamma_pi=100;
par.gamma_T=0.5075;
par.theta_pi=0;

% Initialize System
disp('Computing system for SS.');
F = @(a,b,c,d)Fsys(a,b,c,d,Xss,Yss,Gamma_state,Gamma_control,InvGamma,par,mpar,grid,targets,Copula,P_H,aggrshock,oc);
tic
[Fss,LHS,RHS,Distr] = F(State,State_m,Contr,Contr_m);
toc

% Solve RE via Schmitt-Grohe-Uribe Form
[hx,gx,F1,F2,F3,F4,par] = SGU_solver(F, numstates,numcontrols,Gamma_state,oc,mpar,par,grid);

% Produce IRFs
mpar.maxlag=40; % Quarters

x0=zeros(numstates,1);
x0(end)=par.sigmaS;

MX=[eye(length(x0));gx];
IRF_state_sparse=[];
x=x0;

for t=1:mpar.maxlag
    IRF_state_sparse(:,t)=(MX*x)';
    x=hx*x;
end

plot_IRFs_prep

plot_IRFs

welfare_skript

close all
