function [c_update,m_update]=EGM_Step1_b(grid,inc,money_expense,c_n_aux,par,mpar)
%%EGM_Step1_b computes the optimal consumption and corresponding optimal money 
% holdings in case the capital stock cannot be adjusted by taking the budget 
% constraint into account.
% c_update(m,k,s*h,M,K):    Update for consumption policy under no-adj.
% m_update(m,k,s*h,M,K):    Update for money policy under no-adj.

% Copyright (c) 2018-05-27
% Christian Bayer, Ralph Luetticke, Lien Pham-Dao, and Volker Tjaden

%% EGM: Calculate assets consistent with choices being (m')
% Calculate initial money position from the budget constraint, 
% that leads to the optimal consumption choice
m_star_n = (c_n_aux + money_expense - inc.labor - inc.rent - inc.profits);
m_star_n = (m_star_n<0).*m_star_n./((par.RB+par.borrwedge)./par.PI)...
    + (m_star_n>=0).*m_star_n./((par.RB)./par.PI);
% Identify binding constraints
binding_constraints = money_expense < repmat(m_star_n(1,:,:,:,:),[mpar.nm 1 1 1 1]);

% Consumption when drawing assets m' to zero: Eat all Resources
Resource = inc.labor + inc.rent + inc.money + inc.profits; 

%% Next step: Interpolate w_guess and c_guess from new k-grids
% using c(s,h,k',K), k(s,h,k'K

m_star_n = reshape(m_star_n,[mpar.nm mpar.nk*mpar.ns*mpar.nh*mpar.nM*mpar.nK]);
c_n_aux= reshape(c_n_aux,[mpar.nm mpar.nk*mpar.ns*mpar.nh*mpar.nM*mpar.nK]);

%Interpolate grid.m and c_n_aux defined on m_star_n over grid.m
[c_update, m_update]=egm1b_aux_mex(grid.m,m_star_n,c_n_aux);

c_update = reshape(c_update,[mpar.nm, mpar.nk, mpar.ns*mpar.nh, mpar.nM, mpar.nK]);
m_update = reshape(m_update,[mpar.nm, mpar.nk, mpar.ns*mpar.nh, mpar.nM, mpar.nK]);

c_update(binding_constraints) = Resource(binding_constraints)-grid.m(1);
m_update(binding_constraints) = min(grid.m);

end