function  mstar=EGM_Step2(mutil_c_n,mutil_c_a, psi_guess, grid,P,P_KM,RBRB,RR,QQ,AProb,par,mpar) 
%EGM_Step2 finds the optimal off-grid money choice, m*(k'), given k' (and h, s, M, K)

% Copyright (c) 2018-05-27
% Christian Bayer, Ralph Luetticke, Lien Pham-Dao, and Volker Tjaden

% Here QQ is defined on tomorrows states: q'(s',M',K') and we take the
% expectation to express it in today's states (s,M,K)
term1=((AProb.* mutil_c_a .*(QQ + RR))+((1-AProb).* mutil_c_n .* RR)+(1-AProb).* psi_guess);
term1=expect_mex_new(P,P_KM,par.beta*term1);


% Now QQ is defined on todays states: q(s,M,K)
term2=RBRB.*(AProb.* mutil_c_a +(1-AProb).* mutil_c_n);
term2=expect_mex_new(P,P_KM,par.beta*term2);


% Equation (45) in Appedix A.4.
E_return_diff=term1./QQ-term2;

% Find an m*_n for given k' that solves the difference equation (45)
mstar=Fastroot(grid.m,E_return_diff);
mstar=max(mstar,grid.m(1)); %Use non-negativity constraint and monotonicity
mstar=min(mstar,grid.m(end)); % Do not allow for extrapolation
mstar=reshape(mstar, [mpar.nk,mpar.ns*mpar.nh,mpar.nM,mpar.nK]);
end

function roots = Fastroot(xgrid,fx)
%fast linear interpolation root finding
%(=one Newton step at largest negative function value)
%   stripped down version of interp1 that accepts multiple inputs (max 3)
%   that are interpolated over the same grids x & xi
xgrid=xgrid(:);
fx=reshape(fx,[numel(xgrid),numel(fx)/numel(xgrid)]);

dxgrid=diff(xgrid);
dfx=diff(fx);
idx=ones(1,numel(fx)/numel(xgrid));

% Make use of the fact that the difference equation is monotonically
% increasing in m
idx_min=(fx(1,:)>0); %Corner solutions left (if no solution x* to f(x)=0 exists)
idx_max=(fx(end,:)<0); %Corner solutions right (if no solution x* to f(x)=0 exists)
index=find(and(not(idx_min),not(idx_max))); %interior solutions (if solution x* to f(x)=0 exists)

% Find index of two gridpoints where sign of fx changes from positive to negative,
[~,idx(index)]=max(diff(sign(fx(:,index))));

aux_index=(0:numel(fx)/numel(xgrid)-1)*numel(xgrid); %aux for linear indexes
aux_index2=(0:numel(fx)/numel(xgrid)-1)*(numel(xgrid)-1);    
fxx=fx(idx+aux_index);
xl=xgrid(idx)';
dx=dxgrid(idx)';
dfxx=dfx(idx+aux_index2);
% Because function is piecewise linear in gridpoints, one newton step is
% enough to find the solution 
roots=xl-fxx.*dx./dfxx;

roots(idx_min)=xgrid(1); %constrained choice
roots(idx_max)=xgrid(end); % no-extrapolation
end