function [ c_a_new,m_a_new,k_a_new ] = EGM_Step4( cons_list,res_list,mon_list,cap_list,inc,mpar,grid )
%EGM_Step4 obtains consumption, money, and capital policy under adjustment.
%   The function uses the {(cons_list{j},res_list{j})} as measurement
%   points. The consumption function in (m,k) can be obtained from
%   interpolation by using the total resources available at (m,k): R(m,k)=qk+m/pi.
%
%   c_a_new(m,k,s*h,M,K): Update for consumption policy under adjustment
%   m_a_new(m,k,s*h,M,K): Update for money policy under adjustment
%   k_a_new(m,k,s*h,M,K): Update for capital policy under adjustment

% Copyright (c) 2018-05-27
% Christian Bayer, Ralph Luetticke, Lien Pham-Dao, and Volker Tjaden

c_a_new=NaN([mpar.nm*mpar.nk, mpar.ns*mpar.nh*mpar.nM*mpar.nK]);
m_a_new=NaN([mpar.nm*mpar.nk, mpar.ns*mpar.nh*mpar.nM*mpar.nK]);
k_a_new=NaN([mpar.nm*mpar.nk, mpar.ns*mpar.nh*mpar.nM*mpar.nK]);
Resource_grid=reshape(inc.capital+inc.money+inc.rent,[mpar.nm*mpar.nk, mpar.ns*mpar.nh*mpar.nM*mpar.nK]);
labor_inc_grid=reshape(inc.labor + inc.profits,[mpar.nm*mpar.nk, mpar.ns*mpar.nh*mpar.nM*mpar.nK]);

for j=1:mpar.nM*mpar.nK*mpar.ns*mpar.nh
    log_index=Resource_grid(:,j)<res_list{j}(1);

    % when at most one constraint binds:
    
     [c_a_new(:,j), m_a_new(:,j),k_a_new(:,j)] = ...
        myinter1m_mex(res_list{j},Resource_grid(:,j),cons_list{j},mon_list{j},cap_list{j});
    % Lowest value of res_list corresponds to m_a'=0 and k_a'=0. 
    % Any resources on grid smaller then res_list imply that HHs consume all
    % resources plus income. 
    % When both constraints are binding:
    c_a_new(log_index,j) = Resource_grid(log_index,j) + labor_inc_grid(log_index,j)-grid.m(1); 
    m_a_new(log_index,j) = grid.m(1);
    k_a_new(log_index,j) = 0;
end

c_a_new=reshape(c_a_new,[mpar.nm ,mpar.nk, mpar.ns*mpar.nh,mpar.nM,mpar.nK]);
k_a_new=reshape(k_a_new,[mpar.nm ,mpar.nk, mpar.ns*mpar.nh,mpar.nM,mpar.nK]);
m_a_new=reshape(m_a_new,[mpar.nm ,mpar.nk, mpar.ns*mpar.nh,mpar.nM,mpar.nK]);
end

