function [excess,m_n_prime,m_a_prime,k_a_prime,c_n_prime,c_a_prime,x_n_prime,x_a_prime,AProb,RB,MoneySupply,K_guess,mc,W,R,Y,N,taxrev,Profits,Gov_spend] = ...
ExcessDemand(PriceGuess,M,RB_1,EMU_use,LHS_use,EVa_use,EVn_use,joint_distr,grid,meshes,K,H,SS_interp,k_n_prime,mpar,par,targets,COV_half,COV_inv_half)
%ExcessDemand returns excess demands w.r.t. money and capital given the guess
%for prices and the states {m,k,sxh,M,K} at the beginning of the period.
%In addition, ExcessDemand also returns the current-period policy functions
%given prices and states.

% Copyright (c) 2018-05-27
% Christian Bayer, Ralph Luetticke, Lien Pham-Dao, and Volker Tjaden

util = @(c)(c.^(1-par.xi))./(1-par.xi);

if min(PriceGuess)>0
    PI_guess = PriceGuess(1);
    Q_guess = PriceGuess(2);
    K_guess = ((Q_guess-1)/par.phi+1)*K;

    % Return on bonds
    RB = exp(log(par.RB) + par.rho_R* log(RB_1/par.RB)  + log(PI_guess/par.PI).*((1-par.rho_R)*par.theta_pi));
 
     %% Calculate Rental Rates for factors
    O_next = (log([RB_1 M K])-targets.mean_raw)*COV_inv_half';
    EPIprime =SS_interp'*(par.beta_EPI(1,:)'+ par.beta_EPI(2,:)'.*(O_next(2))+ par.beta_EPI(3,:)'.*(O_next(3))+ par.beta_EPI(4,:)'.*(O_next(1)));

    mc =  par.mu - (par.beta * EPIprime - log(PI_guess))/par.kappa;

    N    =  (par.tau*par.alpha*K.^(1-par.alpha).*mc).^(1/(1-par.alpha+par.gamma));

    W = par.alpha       * mc.* (K./(N)).^(1-par.alpha);

    Y = (N).^(par.alpha).*K.^(1-par.alpha);

    R = (1-par.alpha)   * mc.* ((N)./K).^(par.alpha)  - par.delta;
    Profits = (1-mc).*Y  + 1/2*par.phi*((K_guess-K).^2)./K - Y.*(1/(1-par.mu)/par.kappa)./2 .*log(PI_guess).^2;

    %% Gov Budget constraint
    taxrev =(1-par.tau)*W*N +(1-par.tau)*Profits;

    MoneySupply = targets.B*...
    exp(par.rho_B * log(M/targets.B) ...
    + par.rho_B * log(RB_1/par.RB)...
    - (par.gamma_pi+par.rho_B) * log(PI_guess/par.PI) ...
    - par.gamma_T * log(taxrev/targets.T));

    Gov_spend =  exp((MoneySupply - M./PI_guess)./Y);
    
    meshes.profits=0;%Lump Sum profits

    WW=par.gamma/(1+par.gamma)*(N/H)*W*ones(mpar.nm,mpar.nk,mpar.nh);
    WW(:,:,end)=Profits*par.profitshare;

%% Remove K' by interpolation
    O_next = (log([RB MoneySupply K_guess])-(targets.mean_raw))*COV_inv_half';

    [~,idK]=histc(O_next(3),grid.oK);
    idK(O_next(3)>=grid.oK(end)) = mpar.nK-1;
    idK(O_next(3)<=grid.oK(1))   = 1;

    distK=(O_next(3) - grid.oK(idK))/(grid.oK(idK+1) - grid.oK(idK));

    EMU_use= squeeze(EMU_use(:,:,:,:,:,idK)) + ...
        distK*(squeeze(EMU_use(:,:,:,:,:,idK+1))- squeeze(EMU_use(:,:,:,:,:,idK)));

    LHS_use= squeeze(LHS_use(:,:,:,:,:,idK)) + ...
        distK*(squeeze(LHS_use(:,:,:,:,:,idK+1))- squeeze(LHS_use(:,:,:,:,:,idK)));

    EVa_use= squeeze(EVa_use(:,:,:,:,:,idK)) + ...
        distK*(squeeze(EVa_use(:,:,:,:,:,idK+1))- squeeze(EVa_use(:,:,:,:,:,idK)));

    EVn_use= squeeze(EVn_use(:,:,:,:,:,idK)) + ...
        distK*(squeeze(EVn_use(:,:,:,:,:,idK+1))- squeeze(EVn_use(:,:,:,:,:,idK)));

    %% Remove M by interpolation (express as function of M and PI)
    [~,idM_1]=histc(O_next(2),grid.oM);
    idM_1(O_next(2)>=grid.oM(end)) = mpar.nM2-1;
    idM_1(O_next(2)<=grid.oM(1))   = 1;

    distM_1=(O_next(2) - grid.oM(idM_1))/(grid.oM(idM_1+1) - grid.oM(idM_1));

    EMU_use= squeeze(EMU_use(:,:,:,:,idM_1)) + ...
        distM_1*(squeeze(EMU_use(:,:,:,:,idM_1+1))- squeeze(EMU_use(:,:,:,:,idM_1)));
    LHS_use= squeeze(LHS_use(:,:,:,:,idM_1)) + ...
        distM_1*(squeeze(LHS_use(:,:,:,:,idM_1+1))- squeeze(LHS_use(:,:,:,:,idM_1)));
    EVa_use= squeeze(EVa_use(:,:,:,:,idM_1)) + ...
        distM_1*(squeeze(EVa_use(:,:,:,:,idM_1+1))- squeeze(EVa_use(:,:,:,:,idM_1)));
    EVn_use= squeeze(EVn_use(:,:,:,:,idM_1)) + ...
        distM_1*(squeeze(EVn_use(:,:,:,:,idM_1+1))- squeeze(EVn_use(:,:,:,:,idM_1)));

    [~,idM]=histc(O_next(1),grid.oRB);
    idM(O_next(1)>=grid.oRB(end)) = mpar.nM1-1;
    idM(O_next(1)<=grid.oRB(1))   = 1;

    distM=(O_next(1) - grid.oRB(idM))/(grid.oRB(idM+1) - grid.oRB(idM));

    EMU_use= squeeze(EMU_use(:,:,:,idM)) + ...
        distM*(squeeze(EMU_use(:,:,:,idM+1))- squeeze(EMU_use(:,:,:,idM)));
    LHS_use= squeeze(LHS_use(:,:,:,idM)) + ...
        distM*(squeeze(LHS_use(:,:,:,idM+1))- squeeze(LHS_use(:,:,:,idM)));
    EVa_use= squeeze(EVa_use(:,:,:,idM)) + ...
        distM*(squeeze(EVa_use(:,:,:,idM+1))- squeeze(EVa_use(:,:,:,idM)));
    EVn_use= squeeze(EVn_use(:,:,:,idM)) + ...
        distM*(squeeze(EVn_use(:,:,:,idM+1))- squeeze(EVn_use(:,:,:,idM)));

%% Repeat EGM Steps 1 - 4 for given Q and PI

    %EGM Step 1
    [ m_n_prime,c_new ] = KS_EGM_Step1( EMU_use,RB_1,PI_guess,par.tau*WW,R,meshes,grid,par,mpar );

    % EGM Step 2

    [ m_a_star ] = KS_EGM_Step2( par.beta*LHS_use, par.beta*EMU_use,Q_guess,grid,mpar );

    % EGM Step 3
    [ ~, res_list, mon_list,cap_list  ] = KS_EGM_Step3( par.beta*EMU_use,m_a_star,c_new,Q_guess,par.tau*WW,grid,meshes,mpar,par );

    % EGM Step 4
    [ m_a_prime,k_a_prime ] = KS_EGM_Step4( res_list,mon_list,cap_list,meshes,R,Q_guess,RB_1/PI_guess,mpar,grid );

    x_n_prime = meshes.m*(RB_1/PI_guess) + meshes.k*R + (par.tau*WW.*meshes.h) - m_n_prime + meshes.profits;
    x_a_prime = meshes.m*(RB_1/PI_guess) + meshes.k*(Q_guess+R) + (par.tau*WW.*meshes.h) - m_a_prime - Q_guess*k_a_prime + meshes.profits;

    WW=N*W*ones(mpar.nm,mpar.nk,mpar.nh);
    WW(:,:,end)=Profits*par.profitshare;

    c_n_prime = meshes.m*(RB_1/PI_guess) + meshes.k*R + (par.tau*WW.*meshes.h) - m_n_prime + meshes.profits;
    c_a_prime = meshes.m*(RB_1/PI_guess) + meshes.k*(Q_guess+R) + (par.tau*WW.*meshes.h) - m_a_prime - Q_guess*k_a_prime + meshes.profits;

%% Update Adjustment Prob
    Va =util(x_a_prime) + EVa_use;
    Vn =util(x_n_prime) + EVn_use;

    AProb = max(min(1./(1+exp(-((Va-Vn)-par.mu_chi)./par.sigma_chi)),1-1e-6),1e-6);

%% Calculate Excess Demands

    MoneyDemand = sum(AProb(:).*joint_distr(:).*m_a_prime(:)) + sum((1-AProb(:)).*joint_distr(:).*m_n_prime(:));
    CapitalDemand = sum(AProb(:).*joint_distr(:).*k_a_prime(:)) + sum((1-AProb(:)).*joint_distr(:).*k_n_prime(:));

    excess(1)=MoneyDemand-MoneySupply;
    excess(2)=CapitalDemand-K_guess;

else
    excess=NaN(2,1);
end
