function [ m_a_star ] = KS_EGM_Step2( LHS_use2,EMU_use2,q_guess,grid,mpar )
%KS_EGM_Step2 
% Copyright (c) 2018-05-27
% Christian Bayer, Ralph Luetticke, Lien Pham-Dao, and Volker Tjaden

E_return_diff=LHS_use2./q_guess-EMU_use2;

mstar=Fastroot(grid.m_sim,E_return_diff);
mstar=max(mstar,grid.m(1)); %Use borrowing constraint and monotonicity
mstar=min(mstar,grid.m(end)); % Do not allow for extrapolation
m_a_star=reshape(mstar, [mpar.nk_sim,mpar.nh]);

end
function roots = Fastroot(xgrid,fx)
%fast linear interpolation root finding
%(=one Newton step at largest negative function value)
%   stripped down version of interp1 that accepts multiple inputs (max 3)
%   that are interpolated over the same grids x & xi
xgrid=xgrid(:);
fx=reshape(fx,[numel(xgrid),numel(fx)/numel(xgrid)]);

dxgrid=diff(xgrid);
dfx=diff(fx);
idx=ones(1,numel(fx)/numel(xgrid));

idx_min=(fx(1,:)>0); %Corner solutions left
idx_max=(fx(end,:)<0); %Corner solutions right
index=find(and(not(idx_min),not(idx_max))); %interior solutions

[~,idx(index)]=max(diff(sign(fx(:,index)))); %Find the index of the first change in sign from negative to positive

aux_index=(0:numel(fx)/numel(xgrid)-1)*numel(xgrid); %aux for linear indexes
aux_index2=(0:numel(fx)/numel(xgrid)-1)*(numel(xgrid)-1);    
fxx=fx(idx+aux_index);
xl=xgrid(idx)';
dx=dxgrid(idx)';
dfxx=dfx(idx+aux_index2);
roots=xl-fxx.*dx./dfxx;

roots(idx_min)=xgrid(1); %constrained choice
roots(idx_max)=xgrid(end); % no-extrapolation
end
