%% Load results

beta_q_baseline    = 100*regression(end).betaQ;
beta_q_baseline_R2 = 100*regression(end).regstats(2);
beta_PI_baseline   = 100*regression(end).betaPI;
beta_PI_baseline_R2= 100*regression(end).regstats(1);
beta_EPI_baseline   = 100*regression(end).betaEPI;
beta_EPI_baseline_R2= 100*regression(end).regstats(3);
%% Create Latex Tables - Laws of Motion

filename=['lawsofmotion_KS.tex'];
FID = fopen(filename, 'w');
fprintf(FID, '\\begin{table}[th!] \n');
fprintf(FID, '\\begin{small}  \n');
fprintf(FID, '\\begin{center} \n');
fprintf(FID, '\\begin{threeparttable} \n');

fprintf(FID, '\\caption{Laws of motion for Krusell and Smith}\\label{tab:LOM_KS} \n');
fprintf(FID, ' \\begin{tabular}{l ccccc} \n');

fprintf(FID, '\\\\ \\hline \n');
fprintf(FID, '\\\\ & \\multicolumn{4}{l}{Price of Capital $\\log q_t$} & ($R^2$: %8.2f ) \n', beta_q_baseline_R2);
fprintf(FID, '\\\\ \\hline \n');
fprintf(FID, '\\\\                   & $s_1$ & $s_2$ & $s_3$ & $s_4$ & $s_5$  \\\\ \\cline{2-6} \n');
fprintf(FID, '\\\\ $\\beta^1_{q}$ & %8.2f & %8.2f & %8.2f  & %8.2f & %8.2f   \n', beta_q_baseline(1,:));
fprintf(FID, '\\\\ $\\beta^2_{q}$ & %8.2f & %8.2f & %8.2f  & %8.2f & %8.2f   \n', beta_q_baseline(2,:));
fprintf(FID, '\\\\ $\\beta^3_{q}$ & %8.2f & %8.2f & %8.2f  & %8.2f & %8.2f   \n', beta_q_baseline(3,:));
fprintf(FID, '\\\\ $\\beta^4_{q}$ & %8.2f & %8.2f & %8.2f  & %8.2f & %8.2f   \n', beta_q_baseline(4,:));
fprintf(FID, '\\\\ $\\beta^5_{q}$ & %8.2f & %8.2f & %8.2f  & %8.2f & %8.2f   \n', beta_q_baseline(5,:));
fprintf(FID, '\\\\ \\hline \n');
fprintf(FID, '\\\\ & \\multicolumn{4}{l}{Inflation $\\log \\pi_t$} & ($R^2$: %8.2f ) \n', beta_PI_baseline_R2);
fprintf(FID, '\\\\ \\hline \n');
fprintf(FID, '\\\\                   & $s_1$ & $s_2$ & $s_3$ & $s_4$ & $s_5$  \\\\ \\cline{2-6} \n');
fprintf(FID, '\\\\ $\\beta^1_{\\pi}$ & %8.2f & %8.2f & %8.2f  & %8.2f & %8.2f   \n', beta_PI_baseline(1,:));
fprintf(FID, '\\\\ $\\beta^2_{\\pi}$ & %8.2f & %8.2f & %8.2f  & %8.2f & %8.2f   \n', beta_PI_baseline(2,:));
fprintf(FID, '\\\\ $\\beta^3_{\\pi}$ & %8.2f & %8.2f & %8.2f  & %8.2f & %8.2f   \n', beta_PI_baseline(3,:));
fprintf(FID, '\\\\ $\\beta^4_{\\pi}$ & %8.2f & %8.2f & %8.2f  & %8.2f & %8.2f   \n', beta_PI_baseline(4,:));
fprintf(FID, '\\\\ $\\beta^5_{\\pi}$ & %8.2f & %8.2f & %8.2f  & %8.2f & %8.2f   \n', beta_PI_baseline(5,:));
fprintf(FID, '\\\\ \\hline \n');
fprintf(FID, '\\\\ & \\multicolumn{4}{l}{Expectation term $\\left[\\log(1+\\pi_{t+1})\\tfrac{Y_{t+1}}{Y_t}\\right]$} & ($R^2$: %8.2f ) \n', beta_EPI_baseline_R2);
fprintf(FID, '\\\\ \\hline \n');
fprintf(FID, '\\\\                   & $s_1$ & $s_2$ & $s_3$ & $s_4$ & $s_5$  \\\\ \\cline{2-6} \n');
fprintf(FID, '\\\\ $\\beta^1_{E\\pi}$ & %8.2f & %8.2f & %8.2f  & %8.2f & %8.2f   \n', beta_EPI_baseline(1,:));
fprintf(FID, '\\\\ $\\beta^2_{E\\pi}$ & %8.2f & %8.2f & %8.2f  & %8.2f & %8.2f   \n', beta_EPI_baseline(2,:));
fprintf(FID, '\\\\ $\\beta^3_{E\\pi}$ & %8.2f & %8.2f & %8.2f  & %8.2f & %8.2f   \n', beta_EPI_baseline(3,:));
fprintf(FID, '\\\\ $\\beta^4_{E\\pi}$ & %8.2f & %8.2f & %8.2f  & %8.2f & %8.2f   \n', beta_EPI_baseline(4,:));
fprintf(FID, '\\\\ $\\beta^5_{E\\pi}$ & %8.2f & %8.2f & %8.2f  & %8.2f & %8.2f   \n', beta_EPI_baseline(5,:));

fprintf(FID, '\\\\ \n');

fprintf(FID, '\\\\ \\hline \n');

fprintf(FID, '\\end{tabular} \n');
fprintf(FID, '\\begin{tablenotes} \n');
fprintf(FID, '\\item[1] For readability all values are mutiplied by $100$.  \n');

fprintf(FID, '\\end{tablenotes} \n');
fprintf(FID, '\\end{threeparttable} \n');
fprintf(FID, '\\end{center} \n');
fprintf(FID, '\\end{small}  \n');
fprintf(FID, '\\end{table} \n');

fclose(FID);
