function [PI,EPI,RB_1,RB,q,P_M,P_K,N,R_fc,W_fc,Profits_fc,WW,RR,QQ,PIPI,RBRB,Y,par]=lawsofmotion_2Assets(grid,par,mpar,mode,targets,COV_half,COV_inv_half)
% lawsofmotion_2Assets returns the expected next period values for q and PI
% based on the LoMs.
% Moreover, P_M and P_K are weighting matrices obtained by linear interpolation
% that maps the next period values on the specified grids.
%
% PI(s,M,K):    tomorrow's inflation rate (as KS rule)
% EPI(s,M,k):   tomorrow's expected inflation $E_t\pi_{t+2})$
% q(s,M,K):     tomorrow's price of capital (as KS rule)
% P_M(M,M):          transition matrix for aggregate money
% P_K(K,K):          transition matrix for aggregate capital
% TT(m*k*s*h*M*K,m*k*s*h*M*K):   transition matrix for all states combined

% Copyright (c) 2018-05-27
% Christian Bayer, Ralph Luetticke, Lien Pham-Dao, and Volker Tjaden

P_M=cell(mpar.ns,mpar.nK);
P_K=cell(mpar.ns,mpar.nM);

% Expand up M and K grids and take logs for LoM


M_aux = permute(repmat((grid.oM), [mpar.nM1 1 mpar.ns mpar.nK]),[3 1 2 4]); % log(M)
M     = reshape(M_aux, [mpar.ns mpar.nM1*mpar.nM2 mpar.nK]);

RB_1_aux = permute(repmat((grid.oRB'), [1 mpar.nM2 mpar.ns mpar.nK]), [3 1 2 4]); % log(M(-1))
RB_1     = reshape(RB_1_aux, [mpar.ns mpar.nM1*mpar.nM2 mpar.nK]);

K_aux  = permute(repmat((grid.oK), [mpar.ns 1  mpar.nM1 mpar.nM2]),[1 3 4 2]); % log(K)
K      = reshape(K_aux, [mpar.ns mpar.nM1*mpar.nM2 mpar.nK]);

PI=NaN(mpar.ns,mpar.nM,mpar.nK);
EPI=NaN(mpar.ns,mpar.nM,mpar.nK);
q=NaN(mpar.ns,mpar.nM,mpar.nK);
% Current Inflation and capital price
for ss=1:mpar.ns %PI=f(s,M(-1),M,K) <=> PI'=f(s',M,M',K')
    PI(ss,:,:)    = exp(par.beta_PI(1,ss) + par.beta_PI(2,ss).*M(ss,:,:) + par.beta_PI(3,ss)*K(ss,:,:)   + par.beta_PI(4,ss).*RB_1(ss,:,:));
    EPI(ss,:,:)   = (par.beta_EPI(1,ss)  + par.beta_EPI(2,ss).*M(ss,:,:) + par.beta_EPI(3,ss)*K(ss,:,:)  + par.beta_EPI(4,ss).*RB_1(ss,:,:));
    q(ss,:,:)     = exp(par.beta_q(1,ss)   + par.beta_q(2,ss).*M(ss,:,:) + par.beta_q(3,ss)*K(ss,:,:)    + par.beta_q(4,ss).*RB_1(ss,:,:));
end
[meshes.o1,meshes.o2,meshes.o3] = ndgrid(grid.oRB,grid.oM,grid.oK);
Ostates = [meshes.o1(:) meshes.o2(:) meshes.o3(:)];
Rstates = Ostates*COV_half';
RB_1    = exp(reshape(repmat(Rstates(:,1)',[mpar.ns 1]), [mpar.ns mpar.nM1*mpar.nM2 mpar.nK])+targets.mean_raw(1));
M       = exp(reshape(repmat(Rstates(:,2)',[mpar.ns 1]), [mpar.ns mpar.nM1*mpar.nM2 mpar.nK])+targets.mean_raw(2));
K       = exp(reshape(repmat(Rstates(:,3)',[mpar.ns 1]), [mpar.ns mpar.nM1*mpar.nM2 mpar.nK])+targets.mean_raw(3));


% Implied capital stock
K_next = ((q-1)/par.phi+1).*K;

[ N,R_fc,W_fc,Profits_fc,WW,RR,QQ,PIPI,RBRB,Y] = factor_returns(PI,EPI,RB_1,q,K,K_next,grid,par,mpar);


taxrev =(1-par.tau).*W_fc.*N +(1-par.tau).*Profits_fc;

% Gross nominal return on bonds
RB = exp(log(par.RB) + par.rho_R* log(RB_1/par.RB)  + log(PI/par.PI).*((1-par.rho_R)*par.theta_pi));

M_next = targets.B*...
    exp(par.rho_B * log(M/targets.B) ...
    + par.rho_B * log(RB_1/par.RB)...
    - (par.gamma_pi+par.rho_B) * log(PI/par.PI) ...
    - par.gamma_T * log(taxrev/targets.T));

raw_states   = log([RB(:) M_next(:)  K_next(:)]);
raw_states   = raw_states-repmat(targets.mean_raw, [max(size(raw_states)) 1]);
O_next = raw_states*COV_inv_half';
RB     = reshape(O_next(:,1),[mpar.ns mpar.nM mpar.nK]);
M_next = reshape(O_next(:,2),[mpar.ns mpar.nM mpar.nK]);
K_next = reshape(O_next(:,3),[mpar.ns mpar.nM mpar.nK]);

% transition interpolation matrix for capital
aux=permute(K_next,[3,1,2]); %f(K,s,M)

[~,indexK]=histc(aux,grid.oK);

% Replace the index of out of bounds forecasts (=1) by lower and upper bound for
% the end of interval gridpoints, i.e. 2 for too small forecasts and
% mpar.nM for too large forecasts.
indexK(aux<=grid.oK(1))=1;
indexK(aux>=grid.oK(end))=mpar.nK-1;
weightsK = max(min(1,(aux-grid.oK(indexK))./(grid.oK(indexK+1)-grid.oK(indexK))),0);


for ss=1:mpar.ns
    for MM=1:mpar.nM
        P_K{ss,MM}=zeros(mpar.nK, mpar.nK);
        for KK=1:mpar.nK
            P_K{ss,MM}(KK,indexK(KK,ss,MM)+1) = weightsK(KK,ss,MM)   ;
            P_K{ss,MM}(KK,indexK(KK,ss,MM)) = (1-weightsK(KK,ss,MM));
        end
    end
end


% Transition interpolation for real money
aux=permute(M_next,[2,1,3]); %f(M,s,K)
[~,indexM]=histc(aux,grid.oM);


indexM(aux<grid.oM(1))=1;
indexM(aux>=grid.oM(end))=mpar.nM2-1;
weightsM = max(min(1,(aux-grid.oM(indexM))./(grid.oM(indexM+1)-grid.oM(indexM))),0);


aux=permute(RB,[2,1,3]); %f(M,s,K)
[~,indexRB]=histc(aux,grid.oRB);
indexRB(aux<grid.oRB(1))=1;
indexRB(aux>=grid.oRB(end))=mpar.nM1-1;
weightsRB = max(min(1,(aux-grid.oRB(indexRB))./(grid.oRB(indexRB+1)-grid.oRB(indexRB))),0);


aux=nan(1,mpar.nM2);

for i=1:mpar.nM2
    aux(i)=(i-1)*mpar.nM1;
end

indexRB_MUB=zeros(size(indexM));
indexRB_MLB=zeros(size(indexM));

for ss=1:mpar.ns
    for KK=1:mpar.nK
        P_M{ss,KK}=zeros(mpar.nM, mpar.nM);
        for MM=1:mpar.nM
            indexRB_MUB(MM,ss,KK) = aux(indexM(MM,ss,KK)+1)+indexRB(MM,ss,KK);
            P_M{ss,KK}(MM,indexRB_MUB(MM,ss,KK)+1) = weightsM(MM,ss,KK)*weightsRB(MM,ss,KK);
            P_M{ss,KK}(MM,indexRB_MUB(MM,ss,KK)) = weightsM(MM,ss,KK)*(1-weightsRB(MM,ss,KK));
            
            indexRB_MLB(MM,ss,KK) = aux(indexM(MM,ss,KK))+indexRB(MM,ss,KK);
            P_M{ss,KK}(MM,indexRB_MLB(MM,ss,KK)+1) = (1-weightsM(MM,ss,KK))*weightsRB(MM,ss,KK);
            P_M{ss,KK}(MM,indexRB_MLB(MM,ss,KK)) = (1-weightsM(MM,ss,KK))*(1-weightsRB(MM,ss,KK));
            
        end
    end
end

end
