
%% Use Tauchen method to approximate state space

[P,P_H,P_S,P_HH,P_SH,grid,h_index,par]=stochastics_variance(par, mpar,grid,mode);

KK=[];
while dist.SS > 10*mpar.crit && dist.KS > mpar.crit
    countOL=countOL+1;
    
    %% Prepare items for EGM
    % Layout of matrices:
    %   Dimension 1: money m
    %   Dimension 2: capital k
    %   Dimension 3: stochastic sigma s x stochastic human capital h
    %   Dimension 4: aggregate bonds M
    %   Dimension 5: aggregate capital K
    
    %% 1) Construct relevant returns and interpolation matrices from KS LoMs
    mpar.ns=mpar.nH*mpar.ns;
    
    [PI,EPI,RB_1,RB,q,P_M,P_K,N,R_fc,W_fc,Profits_fc,WW,RR,QQ,PIPI,RBRB,Y,par]=lawsofmotion_2Assets(grid,par,mpar,mode,targets,COV_half,COV_inv_half);
    
    
    %% 3) Guess initial policies and determine labor income
    
    if countOL == 1
        load(SS_name,'AProb')
        AProb=reshape(permute(repmat(AProb(:,:,:),[1 1 1 mpar.ns mpar.nM mpar.nK]),[1 2 4 3 5 6]),[mpar.nm mpar.nk mpar.ns*mpar.nh mpar.nM mpar.nK]);
        
        [c_a_guess,c_n_guess, psi_guess,inc, par]=policyguess(grid,QQ,WW,RR,RBRB,h_index,par,mpar);
        
    else
        [~,~,~,inc,par]=policyguess(grid,QQ,WW,RR,RBRB,h_index,par,mpar);
    end
    
    %% 4) Use EGM to update household policies until convergence
    countz=1;
    dist_AP=1;
    while countz<2 && dist_AP>1e-6
        disp('Solving household problem by EGM')
        tic
        [c_n_guess,m_n_star,c_a_guess,m_a_star,cap_a_star,psi_guess,P_KM]=policies(c_a_guess,c_n_guess,psi_guess,AProb, grid, inc, QQ, RR,RBRB,P,P_K,P_M,mpar,par,mode);
        toc
        tic
        
        [V,EVa,EVn,AProb_new]=ValueFunction(m_n_star,m_a_star,cap_a_star,c_a_guess,c_n_guess,AProb,P,P_KM,mpar,par,grid);
        
        dist_AP=max(abs(AProb_new(:)-AProb(:)))
        AProb=AProb_new;
        countz=countz+1;
        toc
    end
    
    mpar.ns=mpar.ns/mpar.nH;
    
    %% Simulation Krusell Smith
    %   Initialize time series data
    disp('KS simulation')
    % Set parallel pool
    p = gcp('nocreate');
    pool='local'; % select machine
    
    if isempty(p)
        c = parcluster;
        parpool(pool,c.NumWorkers-1)
        p = gcp('nocreate');
    end
    
    sim_mode='LoM';
    seed=123053;
    M_sim  = zeros(mpar.T(1,1),mpar.nblocks);
    ss_sim = zeros(mpar.T(1,1),mpar.nblocks);
    PI_sim = zeros(mpar.T(1,1),mpar.nblocks);
    K_sim  = zeros(mpar.T(1,1),mpar.nblocks);
    q_sim  = zeros(mpar.T(1,1),mpar.nblocks);
    RB_sim = zeros(mpar.T(1,1),mpar.nblocks);
    H_sim = zeros(mpar.T(1,1),mpar.nblocks);
    EPI_sim = zeros(mpar.T(1,1),mpar.nblocks);
    
    
    tic
    parfor tt = 1:mpar.nblocks
        
        [ss_sim(:,tt),M_sim(:,tt),PI_sim(:,tt),EPI_sim(:,tt),K_sim(:,tt),q_sim(:,tt),RB_sim(:,tt),H_sim(:,tt)]=...
            kssimulation_2Assets(c_n_guess,c_a_guess,psi_guess,EVa,EVn,P,P_S,P_H,AProb,joint_distr,grid,QQ,RR,RBRB,tt*seed,mpar,par,targets,tt,sim_mode,COV_half,COV_inv_half);
        
    end
    toc
    
    %% Step 9: Update coefficients of LOM
    
    % Regression inputs
    Ml=((M_sim(mpar.T(2,1):mpar.T(1,1)-2,:)));
    ss=ss_sim(mpar.T(2,1)+1:mpar.T(1,1)-1,:);
    ss=ss(:);
    HH=H_sim(mpar.T(2,1)+1:mpar.T(1,1)-1,:);
    HH=HH(:).^-1;
    ssfut=ss_sim(mpar.T(2,1)+2:mpar.T(1,1),:);
    ssfut=ssfut(:);
    
    Kl=(K_sim(mpar.T(2,1):mpar.T(1,1)-2,:));
    PI=log(PI_sim(mpar.T(2,1)+1:mpar.T(1,1)-1,:));
    PI=PI(:);
    EPI=(EPI_sim(mpar.T(2,1)+1:mpar.T(1,1)-1,:));
    EPI=real(EPI(:));
    q=log(q_sim(mpar.T(2,1)+1:mpar.T(1,1)-1,:));
    q=q(:);
    RBl=RB_sim(mpar.T(2,1):mpar.T(1,1)-2,:);
    keep=(RBl>min(0.98)).*(RBl<max(1.03));
    aux=sum(keep,1);
    aux(aux<max(size(keep)))=0;
    aux(aux==max(size(keep)))=1;
    keep=keep.*repmat(aux,[max(size(keep)) 1]);
    keep=logical(keep(:));
    
    raw_states   = real(log(([RBl(keep) Ml(keep) Kl(keep)])));
    
    if ortho_update
        targets.mean_raw=(mean(raw_states,1));
        raw_states   = raw_states-repmat(targets.mean_raw, [max(size(raw_states)) 1]);
        COV_states   = cov(raw_states);
        [X,L]        = eig(COV_states);
        COV_half     = X*sqrt(L);
        COV_inv_half = diag(1./sqrt(diag(L)))*X';
    else
        raw_states   = raw_states-repmat(targets.mean_raw, [max(size(raw_states)) 1]);
    end
    Ortho_states = raw_states*COV_inv_half';
    
    cov(Ortho_states);
    RAWStates=OStates*COV_half';
    %             grid.RB=exp(RAWStates(:,1)+log(par.theta1));
    %             grid.M=exp(RAWStates(:,2)+log(targets.AggregateMoney));
    %             grid.K=exp(RAWStates(:,3)+log(targets.AggregateCapital));
    
    RB= Ortho_states(:,1);
    M = Ortho_states(:,2);
    K = Ortho_states(:,3);
    
    ss_dummiesmyaux=zeros(size(ss_sim(mpar.T(2,1)+1:mpar.T(1,1)-1,:)));
    ss_dummies=NaN(mpar.ns,size(ss_dummiesmyaux(:),1));
    ss_aux=ss_sim(mpar.T(2,1)+1:mpar.T(1,1)-1,:);
    for j=1:mpar.ns
        ss_dummiesmyaux(ss_aux==j)=1;
        
        ss_dummies(j,:)=ss_dummiesmyaux(:);
        
        ss_dummiesmyaux=zeros(size(ss_aux));
    end
    
    ss_dummiesmyaux=zeros(size(ss_sim(mpar.T(2,1)+2:mpar.T(1,1),:)));
    ss_dummiesfut=NaN(mpar.ns,size(ss_dummiesmyaux(:),1));
    ss_aux=ss_sim(mpar.T(2,1)+2:mpar.T(1,1),:);
    for j=1:mpar.ns
        ss_dummiesmyaux(ss_aux==j)=1;
        
        ss_dummiesfut(j,:)=ss_dummiesmyaux(:);
        
        ss_dummiesmyaux=zeros(size(ss_aux));
    end
    
    ss_dummies=ss_dummies(:,keep);
    ss_dummiesfut=ss_dummiesfut(:,keep);
    
    
    % Regressors
    X_EPI=[ss_dummies(2:mpar.ns,:)' repmat(M,[1 mpar.ns]).*ss_dummies' repmat(K,[1 mpar.ns]).*ss_dummies' repmat(RB,[1 mpar.ns]).*ss_dummies' ss_dummiesfut(2:mpar.ns,:)' HH(keep)];
    
    X=[ss_dummies(2:mpar.ns,:)' repmat(M,[1 mpar.ns]).*ss_dummies' repmat(K,[1 mpar.ns]).*ss_dummies' repmat(RB,[1 mpar.ns]).*ss_dummies' HH(keep)];
    
    % Regressions
    statsPI = regstats(real(PI(keep)),real(X),'linear',{'beta' 'covb' 'rsquare' 'mse' 'tstat'});
    statsEPI = regstats(real(EPI(keep)),real(X_EPI),'linear',{'beta' 'covb' 'rsquare' 'mse' 'tstat'});
    statsQ = regstats(real(q(keep)),real(X),'linear',{'beta' 'covb' 'rsquare' 'mse','r','tstat'});
    
    % Statistics
    R2_PI = statsPI.rsquare;
    R2_Q  = statsQ.rsquare;
    R2_EPI  = statsEPI.rsquare;
    
    beta_ssfut=statsEPI.beta(4*mpar.ns+1:end-1);
    beta_ssfut=[0; beta_ssfut(:)];
    
    Ebeta_ssfut=P_S*beta_ssfut;
    
    clear statsbetaPI_aux statsbetaEPI_aux statsbetaQ_aux
    statsPI.beta(2:mpar.ns)=statsPI.beta(2:mpar.ns)+statsPI.beta(1);
    statsbetaPI_aux(1,:)=statsPI.beta(1:mpar.ns)';
    statsbetaPI_aux(2,:)=statsPI.beta(mpar.ns+1:2*mpar.ns)';
    statsbetaPI_aux(3,:)=statsPI.beta(2*mpar.ns+1:3*mpar.ns)';
    statsbetaPI_aux(4,:)=statsPI.beta(3*mpar.ns+1:4*mpar.ns)';
    
    statsEPI.beta(2:mpar.ns)=statsEPI.beta(2:mpar.ns)+statsEPI.beta(1);
    statsbetaEPI_aux(1,:)=statsEPI.beta(1:mpar.ns)'+Ebeta_ssfut';
    statsbetaEPI_aux(2,:)=statsEPI.beta(mpar.ns+1:2*mpar.ns)';
    statsbetaEPI_aux(3,:)=statsEPI.beta(2*mpar.ns+1:3*mpar.ns)';
    statsbetaEPI_aux(4,:)=statsEPI.beta(3*mpar.ns+1:4*mpar.ns)';
    
    statsQ.beta(2:mpar.ns)=statsQ.beta(2:mpar.ns)+statsQ.beta(1);
    statsbetaQ_aux(1,:)=statsQ.beta(1:mpar.ns)';
    statsbetaQ_aux(2,:)=statsQ.beta(mpar.ns+1:2*mpar.ns)';
    statsbetaQ_aux(3,:)=statsQ.beta(2*mpar.ns+1:3*mpar.ns)';
    statsbetaQ_aux(4,:)=statsQ.beta(3*mpar.ns+1:4*mpar.ns)';
    
    statsbetaEPI=statsbetaEPI_aux;
    statsbetaEPI(5,:)=statsEPI.beta(end);
    statsbetaPI=statsbetaPI_aux;
    statsbetaPI(5,:)=statsPI.beta(end);
    statsbetaQ=statsbetaQ_aux;
    statsbetaQ(5,:)=statsQ.beta(end);
    
    statsbetaQ_aux=repmat(statsbetaQ_aux,[1 mpar.nH]);
    statsbetaPI_aux=repmat(statsbetaPI_aux,[1 mpar.nH]);
    statsbetaEPI_aux=repmat(statsbetaEPI_aux,[1 mpar.nH]);
    for j=1:mpar.nH
        statsbetaQ_aux(1,1+(j-1)*mpar.ns:j*mpar.ns)=statsbetaQ_aux(1,1+(j-1)*mpar.ns:j*mpar.ns)+grid.H(j)*statsQ.beta(end);
        statsbetaPI_aux(1,1+(j-1)*mpar.ns:j*mpar.ns)=statsbetaPI_aux(1,1+(j-1)*mpar.ns:j*mpar.ns)+grid.H(j)*statsPI.beta(end);
        statsbetaEPI_aux(1,1+(j-1)*mpar.ns:j*mpar.ns)=statsbetaEPI_aux(1,1+(j-1)*mpar.ns:j*mpar.ns)+grid.H(j)*statsEPI.beta(end);
    end
    
    
    % Save coefficients on which the time series was run and also
    % the
    % updates as well as the time series itself
    regression(countOL).par  = par; %#ok<*SAGROW>
    regression(countOL).betaQ = statsbetaQ;
    regression(countOL).betaPI = statsbetaPI;
    regression(countOL).betaEPI = statsbetaEPI;
    %             regression(countOL).Y    = [M K PI q];
    regression(countOL).regstats  = [R2_PI; R2_Q; R2_EPI];
    save([filename], 'regression','grid')
    
    % Check convergence of LoM coefficients
    distKS_Q = max(max(abs(statsbetaQ_aux-par.beta_q)))
    distKS_PI = max(max(abs(statsbetaPI_aux-par.beta_PI)))
    distKS_EPI = max(max(abs(statsbetaEPI_aux-par.beta_EPI)))
    dist.KS=max(max(distKS_EPI,distKS_PI),distKS_Q);
    
    %%
    
    KS_par     = [par.beta_PI(:)', par.beta_EPI(:)', par.beta_q(:)']; % x_n
    KS_par_reg = [real(statsbetaPI_aux(:))', real(statsbetaEPI_aux(:))', real(statsbetaQ_aux(:))'];
    
    difference_par = KS_par-KS_par_reg;  % f_{n}
    rel_change = (-Jinv*difference_par')./(abs(KS_par')+.1);
    a=min(.25,mpar.weightlastguess/norm(rel_change));
    update_par = KS_par'-a*Jinv*difference_par';
    update_par = reshape(update_par,length(par.beta_PI(:)),3);
    
    switch update
        case 'Boyden'
            par.beta_PI  = reshape(update_par(:,1),4,mpar.ns);%(1-a)*real(statsbetaPI_aux)+(a)*par.beta_PI;
            par.beta_EPI = reshape(update_par(:,2),4,mpar.ns);%(1-a)*real(statsbetaEPI_aux)+(a)*par.beta_EPI;
            par.beta_q   = reshape(update_par(:,3),4,mpar.ns);%(1-a)*real(statsbetaQ_aux)+(a)*par.beta_q;
            
        case 'secant'
            a=mpar.weightlastguess;
            
            par.beta_PI  = (1-a)*real(statsbetaPI_aux)+(a)*par.beta_PI;
            par.beta_EPI  = (1-a)*real(statsbetaEPI_aux)+(a)*par.beta_EPI;
            par.beta_q   = (1-a)*real(statsbetaQ_aux)+(a)*par.beta_q;
            
            
    end
    %%
    %% Jacobian update
    if countOL>2
        delta_x = (KS_par - KS_par_old)'; % needs to be a column vector
        delta_f = difference_par - difference_par_old;
        Jinv = Jinv + ((delta_x - Jinv*delta_f')*(delta_x'*Jinv))/(delta_x'*Jinv*delta_f');
        % BFGS formula
        %                Jinv = Jinv + (delta_x' * delta_f' + delta_f *Jinv *delta_f' )*(delta_x*delta_x')/((delta_x'*delta_f').^2)...
        %                            - Jinv*(delta_f'*delta_x' + delta_x*delta_f *Jinv)/(delta_x'*delta_f');
    end
    if countOL==20
        ortho_update='false';
        mpar.T(1,1)     = 250; % number of periods
        mpar.T(2,1)     = 50;  % burn-in phase
    end
    difference_par_old = difference_par; % f_{n-1}
    KS_par_old = KS_par; %x_{n-1}
    
    
end

%% Save results
KS_name=[SS_name  '_' date '_' num2str(starttime(4)) 'h' num2str(starttime(5)) 'm'];
save(['Results_KS_' KS_name],'KS_name','AProb','grid','mpar','par','regression','targets','starttime','seed','c_a_guess','c_n_guess', 'psi_guess','inc','COV_half','COV_inv_half','-v7.3')

LOM_table

%% Simulate IRFs
tic
sim_mode='interpolate';
mpar.nblocks=1;
mpar.T(1,1)=10;
mpar.T(2,1)=5;

mpar.shock=0;

M_sim_0  = zeros(mpar.T(1,1),mpar.nblocks);
ss_sim_0 = zeros(mpar.T(1,1),mpar.nblocks);
PI_sim_0 = zeros(mpar.T(1,1),mpar.nblocks);
EPI_sim_0 = zeros(mpar.T(1,1),mpar.nblocks);
K_sim_0  = zeros(mpar.T(1,1),mpar.nblocks);
q_sim_0  = zeros(mpar.T(1,1),mpar.nblocks);
RB_sim_0 =zeros(mpar.T(1,1),mpar.nblocks);
H_sim_0 =zeros(mpar.T(1,1),mpar.nblocks);
D_sim_0 =zeros(mpar.T(1,1),mpar.nblocks);
W_sim_0 =zeros(mpar.T(1,1),mpar.nblocks);
MC_sim_0 =zeros(mpar.T(1,1),mpar.nblocks);
TaxRev_sim_0 =nan(mpar.T(1,1),mpar.nblocks);
BC_sim_0 =nan(mpar.T(1,1),mpar.nblocks);
Profits_sim_0 =nan(mpar.T(1,1),mpar.nblocks);
Y_sim_0 =zeros(mpar.T(1,1),mpar.nblocks);
C_sim_0 =zeros(3,mpar.T(1,1),mpar.nblocks);
I_sim_0 =zeros(mpar.T(1,1),mpar.nblocks);
G_sim_0 =zeros(mpar.T(1,1),mpar.nblocks);
N_sim_0 =zeros(mpar.T(1,1),mpar.nblocks);
AP_sim_0 =zeros(mpar.T(1,1),mpar.nblocks);
Ginis_sim_0 =zeros(3,mpar.T(1,1),mpar.nblocks);
lambdaLI_0=zeros(4,100,mpar.nblocks);
lambdaIL_0=zeros(4,100,mpar.nblocks);
lambdaC_0=zeros(4,100,mpar.nblocks);
lambdaX_0=zeros(4,100,mpar.nblocks);

for tt = 1:mpar.nblocks
    
    [ss_sim_0(:,tt),M_sim_0(:,tt),PI_sim_0(:,tt),EPI_sim_0(:,tt),K_sim_0(:,tt),q_sim_0(:,tt),RB_sim_0(:,tt),H_sim_0(:,tt),D_sim_0(:,tt),...
        W_sim_0(:,tt),MC_sim_0(:,tt),TaxRev_sim_0(:,tt),Y_sim_0(:,tt),C_sim_0(:,:,tt),I_sim_0(:,tt),...
        G_sim_0(:,tt),N_sim_0(:,tt),BC_sim_0(:,tt),Profits_sim_0(:,tt),AP_sim_0(:,tt),lambdaLI_0(:,:,tt),lambdaIL_0(:,:,tt),lambdaC_0(:,:,tt),lambdaX_0(:,:,tt)]=...
        kssimulation_2Assets_IRF(c_n_guess,c_a_guess,psi_guess,EVa,EVn,P,P_S,P_H,AProb,joint_distr,grid,QQ,RR,RBRB,tt*seed,mpar,par,targets,tt,sim_mode,COV_half,COV_inv_half);
    
end

mpar.shock=1;

M_sim_1  = zeros(mpar.T(1,1),mpar.nblocks);
ss_sim_1 = zeros(mpar.T(1,1),mpar.nblocks);
PI_sim_1 = zeros(mpar.T(1,1),mpar.nblocks);
EPI_sim_1 = zeros(mpar.T(1,1),mpar.nblocks);
K_sim_1  = zeros(mpar.T(1,1),mpar.nblocks);
q_sim_1  = zeros(mpar.T(1,1),mpar.nblocks);
RB_sim_1 =zeros(mpar.T(1,1),mpar.nblocks);
H_sim_1 =zeros(mpar.T(1,1),mpar.nblocks);
D_sim_1 =zeros(mpar.T(1,1),mpar.nblocks);
W_sim_1 =zeros(mpar.T(1,1),mpar.nblocks);
MC_sim_1 =zeros(mpar.T(1,1),mpar.nblocks);
TaxRev_sim_1 =nan(mpar.T(1,1),mpar.nblocks);
BC_sim_1 =nan(mpar.T(1,1),mpar.nblocks);
Profits_sim_1 =nan(mpar.T(1,1),mpar.nblocks);
Y_sim_1 =zeros(mpar.T(1,1),mpar.nblocks);
C_sim_1 =zeros(3,mpar.T(1,1),mpar.nblocks);
I_sim_1 =zeros(mpar.T(1,1),mpar.nblocks);
G_sim_1 =zeros(mpar.T(1,1),mpar.nblocks);
N_sim_1 =zeros(mpar.T(1,1),mpar.nblocks);
AP_sim_1 =zeros(mpar.T(1,1),mpar.nblocks);
Ginis_sim_1 =zeros(3,mpar.T(1,1),mpar.nblocks);
lambdaLI_1=zeros(4,100,mpar.nblocks);
lambdaIL_1=zeros(4,100,mpar.nblocks);
lambdaC_1=zeros(4,100,mpar.nblocks);
lambdaX_1=zeros(4,100,mpar.nblocks);


for tt = 1:mpar.nblocks
    
    [ss_sim_1(:,tt),M_sim_1(:,tt),PI_sim_1(:,tt),EPI_sim_1(:,tt),K_sim_1(:,tt),q_sim_1(:,tt),RB_sim_1(:,tt),H_sim_1(:,tt),D_sim_1(:,tt),...
        W_sim_1(:,tt),MC_sim_1(:,tt),TaxRev_sim_1(:,tt),Y_sim_1(:,tt),C_sim_1(:,:,tt),I_sim_1(:,tt),...
        G_sim_1(:,tt),N_sim_1(:,tt),BC_sim_1(:,tt),Profits_sim_1(:,tt),AP_sim_1(:,tt),lambdaLI_1(:,:,tt),lambdaIL_1(:,:,tt),lambdaC_1(:,:,tt),lambdaX_1(:,:,tt)]=...
        kssimulation_2Assets_IRF(c_n_guess,c_a_guess,psi_guess,EVa,EVn,P,P_S,P_H,AProb,joint_distr,grid,QQ,RR,RBRB,tt*seed,mpar,par,targets,tt,sim_mode,COV_half,COV_inv_half);
end

toc
generate_IRFs_relative

plot_IRFs_KS_comparison
