function [c_n_new,m_n_star,c_a_new,m_a_star,cap_a_star,psi_new,P_KM] = policies(c_a_guess,c_n_guess,psi_guess,AProb, ...
    grid, inc, QQ, RR,RBRB,P,P_K,P_M,mpar,par,mode)
%policies solves for the household policies for consumption, money and
%capital holdings in the adjustment and non-adjustment case. Given returns
%{q,R_fc,PI} and interpolation matrices {P,P_K,P_M,TT} a variant of the
%endogeneous gridpoint method is used to iterate on the first order
%conditions until convergence.
%Policies in adjustment case:
%{c_a_new,m_a_star,cap_a_star}
%Policies in non-adjustment case:
%{c_n_new,m_n_star}

% Copyright (c) 2018-05-27
% Christian Bayer, Ralph Luetticke, Lien Pham-Dao, and Volker Tjaden

%% Array version of P_M and P_K interpolation matrices for use in MEX-files
P_M_large=NaN(mpar.nM,mpar.nM,mpar.ns,mpar.nK);
for ii=1:mpar.ns
    for jj=1:mpar.nK
        P_M_large(:,:,ii,jj)=P_M{ii,jj};
    end
end

P_K_large=NaN(mpar.nK,mpar.nK,mpar.ns,mpar.nM);
for ii=1:mpar.ns
    for jj=1:mpar.nM
        P_K_large(:,:,ii,jj)=P_K{ii,jj};
    end
end

P_KM=zeros(mpar.nM*mpar.nK,mpar.nM*mpar.nK,mpar.ns);
for i1=1:mpar.nK
    for i2=1:mpar.nM
        for i3=1:mpar.ns
            for Knext=1:mpar.nK
                kprob=P_K_large(i1,Knext,i3,i2);
                for Mnext=1:mpar.nM
                    mprob=P_M_large(i2,Mnext,i3,i1);
                    P_KM(i2 + (i1-1)*mpar.nM,Mnext + (Knext-1)*mpar.nM,i3)=mprob*kprob;
                end
            end
        end
    end
end

%% Apply EGM to solve for optimal policies and marginal utilities
money_expense  = repmat(grid.m'*ones(1, mpar.nk),[1 1 mpar.nh*mpar.ns mpar.nM mpar.nK]);
distC_n = 99999;
distPSI = distC_n;
distC_a = distPSI;
mutil_c_n = 1./(c_n_guess.^par.xi); % marginal utility at consumption policy no adjustment
mutil_c_a = 1./(c_a_guess.^par.xi); % marginal utility at consumption policy adjustment
mutil_c = AProb.*mutil_c_a + (1-AProb).*mutil_c_n; % Expected marginal utility at consumption policy (w &w/o adjustment)

count=0;
while max([distC_n distC_a distPSI])>mpar.crit && count<200
    count=count+1;
    
    %% Step 1: Update policies for only money adjustment
    [c_n_aux,EMU_aux]=EGM_Step1_a(mutil_c,P,RBRB,P_KM,par,mpar);
    
    % Take borrowing constraint into account
    [c_n_new,m_n_star]=EGM_Step1_b(grid,inc,money_expense,c_n_aux,par,mpar);
    m_n_star(m_n_star>grid.m(end))=grid.m(end);
    
    %% Step 2: Find for every k on grid some off-grid m*(k')
    m_a_star_aux=EGM_Step2(mutil_c_n,mutil_c_a, psi_guess, grid,P,P_KM,RBRB,RR,QQ,AProb,par,mpar);
    
    %% Step 3: Solve for initial resources / consumption in  adjustment case
    [ ~,~,cons_list, res_list, mon_list,cap_list ] = EGM_Step3(EMU_aux,grid,inc,m_a_star_aux,c_n_aux,mpar,par);
    
    %% Step 4: Interpolate Consumption Policy
    [ c_a_new, m_a_star,cap_a_star ] = EGM_Step4( cons_list,res_list, mon_list,cap_list,inc,mpar,grid );
    m_a_star(m_a_star>grid.m(end))=grid.m(end);
    cap_a_star(cap_a_star>grid.k(end))=grid.k(end);
    %% Step 5: Update ~psi
    mutil_c_n = 1./(c_n_new.^par.xi); % marginal utility at consumption policy no adjustment
    mutil_c_a = 1./(c_a_new.^par.xi); % marginal utility at consumption policy adjustment
    mutil_c_n(mutil_c_n==Inf)=1e16;
    mutil_c_a(mutil_c_a==Inf)=1e16;
    mutil_c = AProb.*mutil_c_a + (1-AProb).*mutil_c_n; % Expected marginal utility at consumption policy (w &w/o adjustment)
    
    %% VFI analogue in updating psi
    if count<3
        E_rhs_psi = EGM_Step5(mutil_c_n,mutil_c_a,psi_guess,P,P_KM,QQ,RR,AProb,par,mpar);
        
        E_rhs_psi=reshape(E_rhs_psi, [mpar.nm mpar.nk*mpar.ns*mpar.nh*mpar.nM*mpar.nK]);
        m_n_star=reshape(m_n_star, [mpar.nm mpar.nk*mpar.ns*mpar.nh*mpar.nM*mpar.nK]);
        
        % Interpolation of psi-function at m*_n(m,k)
        [~,idx]=histc(m_n_star,grid.m); % find indexes on grid next smallest to optimal policy
        idx(m_n_star<=grid.m(1))=1; %if below minimum
        idx(m_n_star>=grid.m(end))=mpar.nm-1; %if above maximum
        step=diff(grid.m); %Stepsize on grid
        s = (m_n_star - grid.m(idx))./step(idx);%Distance of optimal policy to next grid point
        
        aux_index=ones(mpar.nm,1)*(0:(mpar.nk*mpar.ns*mpar.nh*mpar.nM*mpar.nK)-1)*mpar.nm; %aux for linear indexes
        aux3=E_rhs_psi(idx(:)+aux_index(:)); %calculate linear indexes
        
        psi_new = aux3 + s(:).*(E_rhs_psi(idx(:)+aux_index(:)+1)-aux3); %linear interpolation
        psi_new = reshape(psi_new,[mpar.nm, mpar.nk, mpar.ns*mpar.nh,mpar.nM,mpar.nK]);
        m_n_star=reshape(m_n_star, [mpar.nm mpar.nk mpar.ns*mpar.nh mpar.nM mpar.nK]);
        distPSI = max((abs(psi_guess(:)-psi_new(:))));
        %% PFI analogue in updating psi
    else
        REP=6;
        E_rhs_mu2 = EGM_Step5a(mutil_c_n,mutil_c_a,P,P_KM,QQ,RR,AProb,par,mpar);
        
        m_n_star=reshape(m_n_star, [mpar.nm mpar.nk*mpar.ns*mpar.nh*mpar.nM*mpar.nK]);
        
        [~,idm_n]=histc(m_n_star,grid.m); % find indexes on grid next smallest to optimal policy
        idm_n(m_n_star<=grid.m(1))=1; %if below minimum
        idm_n(m_n_star>=grid.m(end))=mpar.nm-1; %if above maximum
        
        psi_new= EGM_Step5bK_BLAS_new_mex(E_rhs_mu2,m_n_star,psi_guess,grid.m,...
            int32(idm_n),P,P_KM,par.beta,AProb,int32(REP));
        
        m_n_star=reshape(m_n_star, [mpar.nm mpar.nk mpar.ns*mpar.nh mpar.nM mpar.nK]);
        distPSI = max((abs(psi_guess(:)-psi_new(:))));
        
    end
    %% Step 6: Check convergence of policies
    distC_n = max((abs(c_n_guess(:)-c_n_new(:))));
    distC_a = max((abs(c_a_guess(:)-c_a_new(:))));
    
    % Update c policy guesses
    c_n_guess=c_n_new;
    c_a_guess=c_a_new;
    psi_guess=psi_new;
    
end

disp(([distC_n distC_a distPSI]));

