function [c_a_guess,c_n_guess, psi_guess, inc,par]=policyguess(grid,QQ,WW,RR,RBRB,h_index,par,mpar)
%policyguess returns autarky policy guesses (in the first period only):
% c_a_guess, c_n_guess, psi_guess
% as well as income matrices later on used in EGM: 
% inc 
%
% Consumption is compositite leisure and physical consumption (x_it) in the
% paper, therefore labor income is reduced by the fraction of leisure
% consumed.

% Copyright (c) 2018-05-27
% Christian Bayer, Ralph Luetticke, Lien Pham-Dao, and Volker Tjaden

inc.labor   = par.tau.*WW.*(permute(repmat(grid.h(h_index)',[1 mpar.nm mpar.nk mpar.nM mpar.nK]),[2 3 1 4 5])); 
inc.rent    = RR.*repmat(ones(mpar.nm,1)*grid.k,[1 1 mpar.nh*mpar.ns mpar.nM mpar.nK]);
inc.money   = RBRB.*repmat(grid.m'*ones(1, mpar.nk),[1 1 mpar.nh*mpar.ns mpar.nM mpar.nK]);
inc.capital = QQ.*repmat(ones(mpar.nm,1)*grid.k,[1 1 mpar.nh*mpar.ns mpar.nM mpar.nK]);
inc.profits = 0;% Lump-sum PROFITS;

%% Initial policy guesses:  Autarky policies as guess

% Consumption guess
c_a_guess = inc.labor + inc.rent + inc.capital + max(inc.money,0) + inc.profits;
c_n_guess = inc.labor + inc.rent               + max(inc.money,0) + inc.profits;

% Initially guessed marginal continuation value of holding capital
psi_guess      = zeros([mpar.nm mpar.nk mpar.ns*mpar.nh mpar.nM mpar.nK]);

end



