function [P,P_H,P_S,P_HH,P_SH,grid,h_index,par]=stochastics_variance(par, mpar, grid,mode)
% stochastics_variance generates transition probabilities for:
% h: P_H,
% s: P_S,
% sxh: P
% and hgrid, sgrid on which the probability matrices are defined.
% Also an index function INDEX_H is created to read the individual
% productivity state h from the sxh state.

% Copyright (c) 2018-05-27
% Christian Bayer, Ralph Luetticke, Lien Pham-Dao, and Volker Tjaden

% Generate transition probabilities and grid
% [P_S, sgrid] = rouwen(par.rhoS, 0, par.sigmaS/sqrt(1-par.rhoS^2), mpar.ns);

% Generate transition probabilities and grid
[sgrid,P_S] = Tauchen(par.rhoS,mpar.ns,1, 0, 'importance'); % LR variance = 1
 sgrid      = sgrid*par.sigmaS/sqrt(1-par.rhoS^2);

grid.s       = (exp(sgrid))/mean(exp(sgrid));
%factor       = (1-par.rhoS)*(1-par.rhoH^2);
%mean_HH      = par.sigmaH^2/(1-par.rhoH^2);
%phi1         = par.rhoH^2 + par.rhoS;
%phi2         = - par.rhoH^2*par.rhoS;
%std_HH       = par.sigmaH*sqrt(1 - phi1^2*(1+phi2)/(1-phi2) - phi2^2);
maxHH        = (1+1-grid.s(1))*log(par.H);
minHH        = grid.s(1)*log(par.H);
grid.HH      = linspace(-maxHH,-minHH,mpar.nH);%1./(1-par.rhoH^2)+linspace(-2*par.sigmaS/sqrt(factor^2), 2*par.sigmaS/sqrt(factor^2),mpar.nH);
[mH,ms]      = ndgrid(grid.HH,(-grid.s'));
Hplus        = par.rhoH^2*mH + par.sigmaH^2/2 * ms;
Hplus        = exp(Hplus);
grid.H       = exp(grid.HH);%+(par.H-exp(mean_HH/2));
[Dist_H,idH] = genweight(Hplus,grid.H);
P_HH         = zeros(mpar.nH,mpar.nH,mpar.ns);
for j=1:mpar.ns
    for i=1:mpar.nH
        P_HH(i,idH(i,j)+1,j) = Dist_H(i,j);
        P_HH(i,idH(i,j),j)   = 1-Dist_H(i,j);
    end
end

% load H_transitionmatrix P_HH

% First for human capital
% Generate transition probabilities and grid
[hgrid,P_H,boundsH] = Tauchen(par.rhoH,mpar.nh-1,1, 0, 'importance'); % LR variance = 1
% Correct long run variance for *human capital*
switch(mode)
    case('SS')
    hgrid               = hgrid*par.sigmaH/sqrt(1-par.rhoH^2);
    hgrid               = exp(hgrid); % Levels instead of Logs

    grid.h=[hgrid 1];
end

P_H=[P_H repmat(mpar.in,[mpar.nh-1 1])];
lastrow=[repmat(0,[1, mpar.nh-1]) 1-mpar.out];
lastrow(ceil(mpar.nh/2))=mpar.out;
P_H=[P_H; lastrow];


% Time varying risk is induced as a constant grid with time varying
% transition probabilities
P_aux = repmat (P_H,[1 1 mpar.ns]);

% grid.s=grid.s./grid.shigh;
switch(mode)
    case('SS')
    grid.s=ones(mpar.ns,1)'*grid.s(ceil(mpar.ns/2));
end

    for j = 1:mpar.ns
        aux             = sqrt(grid.s(j))*sqrt(1-par.rhoH^2); % Short run std
        P_aux(1:end-1,1:end-1,j)      = transition(mpar.nh-1,par.rhoH,aux,boundsH);
    end
    P_aux=P_aux./repmat(sum(P_aux,2),[1 size(P_aux,2)]);

switch(mode)
    case('SS')
    Paux=P_H(:,:,ceil(mpar.ns/2))^1000^1000;
    hh=Paux(1,1:mpar.nh-1)*grid.h(1:mpar.nh-1)';
    par.H=hh(1); %Total Employment
    par.profitshare=Paux(end,end)^-1;
end
%% Combine the two stochastic states
% s=1, h=1...nh
%
% s=2, h=1...nh etc

% Define indeces for converting joint states to individual ones
h_index=permute(repmat((1:mpar.nh),mpar.ns,1,mpar.nH),[1 3 2]);
h_index=h_index(:); % maps joint states to h dimension

% Calculate transition probability matrix for joint process
% Calculate transition probability matrix for joint process
P=NaN(mpar.ns*mpar.nH*mpar.nh);
for hh=1:mpar.nh
    for HH=1:mpar.nH
        for s=1:mpar.ns
            P(s + (HH-1)*mpar.ns+(hh-1)*mpar.ns*mpar.nH,:)=kron(kron(P_aux(hh,:,s),P_HH(HH,:,s)),P_S(s,:));
        end
    end
end

P=P./repmat(sum(P,2),[1 size(P,2)]);

P_SH=NaN(mpar.ns*mpar.nH);
    for HH=1:mpar.nH
        for s=1:mpar.ns
            P_SH(s + (HH-1)*mpar.ns,:)=kron(P_HH(HH,:,s),P_S(s,:));
        end
    end

P_SH=P_SH./repmat(sum(P_SH,2),[1 size(P_SH,2)]);

P_H=P_aux;

function [ weight,index ] = genweight( x,xgrid )
% function: GENWEIGHT generates weights and indexes used for linear interpolation
%
% X: Points at which function is to be interpolated.
% xgrid: grid points at which function is measured
% no extrapolation allowed
[~,index] = histc(x,xgrid);
index(x<=xgrid(1))=1;
index(x>=xgrid(end))=length(xgrid)-1;

weight = (x-xgrid(index))./(xgrid(index+1)-xgrid(index)); % weight xm of higher gridpoint
% weight(weight<=0) = 1.e-16; % no extrapolation
% weight(weight>=1) = 1-1.e-16; % no extrapolation

end  % funct
end
