%% ************************************************************************
%
% Matlab code to accompany the paper
% 'Precautionary Savings, Illiquid Assets, and the Aggregate Consequences of
%  Shocks to Household Income Risk', Bonn mimeo
% http://wiwi.uni-bonn.de/hump/wp.html
% by
% Christian Bayer (christian.bayer@uni-bonn.de),
% Ralph Luetticke, Lien Pham-Dao, and Volker Tjaden
%
% Copyright (c) 2018 Christian Bayer, Ralph Luetticke, Lien Pham-Dao, and Volker Tjaden
% =========================================================================
%
% Permission is hereby granted, free of charge, to any person obtaining a
% copy of this software and associated documentation files (the "Software"),
% to deal in the Software without restriction, including without limitation
% the rights to use, copy, modify, merge, publish, distribute, sublicense,
% and/or sell copies of the Software, and to permit persons to whom the
% Software is furnished to do so, subject to the following conditions:
%
% The most recent version or successor of the above-mentioned paper is
% properly cited in all work and publications that benefit from insights
% derived from the Software.
% The above copyright notice and this permission notice shall be included
% in all copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
% THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
% DEALINGS IN THE SOFTWARE.
%
% =========================================================================
%% ************************************************************************
%
% File content:
% Solve the household planning problem with two assets and find
% the Krusell-Smith equilibrium.
% by Christian Bayer, Ralph L�tticke, Lien Pham-Dao, and Volker Tjaden

%% Initialize workspace and load directories
clear
clc
close all
Computername='NKIM'
starttime=clock;
set(0,'defaulttextinterpreter','latex')
addpath(genpath('functions'))
addpath(genpath('latex'))

% Krusell-Smith (KS)
mode='KS';

% Initialize LoMs for q, PI, and EPI
LOMguess='steadystate';
% LOMguess='Reiter';
% LOMguess='LoadPrevious';

% Set updates
update='secant';
ortho_update=true;

%% Load parameters and starting guesses for law of motions
SS_name=['../Baseline/SS_BASELINE_KS.mat'];
load(SS_name,'joint_distr','targets','par','mpar','grid','P_H','AProb');
index_h=(0:mpar.ns:(mpar.nh-1)*mpar.ns) +2;

defineKS_pars
switch LOMguess
    case 'LoadPrevious'
        load('','AProb','regression','c_a_guess','c_n_guess', 'psi_guess','COV_half','COV_inv_half');
        par=regression(end).par;
    case 'Reiter'
        load('../Baseline/LoM_Reiter_Ortho.mat')
        par=paraux;
end
clear regression paraux

OStates=[grid.oRB(:) grid.oM(:) grid.oK(:)];
RAWStates=OStates*COV_half';
grid.RB=exp(RAWStates(:,1)+log(par.RB));
grid.M=exp(RAWStates(:,2)+log(targets.B));
grid.K=exp(RAWStates(:,3)+log(targets.K));
targets.mean_raw=log([par.RB targets.B targets.K]);

Jinv=eye(3*4*mpar.ns*mpar.nH);
filename=['regression_2Assets_' date '_' num2str(starttime(4)) 'h' num2str(starttime(5)) 'm'];

%% Prepare Equilibrium Finding Loop

% Steady state loop on aggr. capital and money
dist.SS  = 99999;
% Krusell-Smith loop on the coefficients of the LoM
dist.KS  = 1e4;
countOL  = 0;    % Count iterations on the outer loop (KS or SS)

mainskript
